<?php

require_once("lib-action/action.php");

use Billing\Manager\ContactManager;

class EditClientContactResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$contact = Factory()->Contact(bgpc("clientContactID"));

		$contactFields = array(
			"contact_active",
			"contact_company",
			"contact_first_name",
			"contact_last_name",
			"contact_email",
			"contact_username",
			"contact_address_1",
			"contact_address_2",
			"contact_city",
			"contact_state",
			"contact_county",
			"contact_zip",
			"countries_id",
			"language_id",
			"contact_phone1",
			"contact_phone2",
			"contact_fax",
			"contact_mobile_phone",
			"contact_email_format",
			"contact_use_specified_email_rules_only",
		);

		$validation_fields = array(
			"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactEmail" => INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED,
			"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactCompany" => INPUT_RESTRICTION_STRING,
			"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactAddress2" => INPUT_RESTRICTION_STRING,
			"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactState" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
			"contactPhone2" => INPUT_RESTRICTION_PHONE,
			"contactFax" => INPUT_RESTRICTION_PHONE,
			"contactPrimary" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
			"contactCounty" => INPUT_RESTRICTION_STRING,
			"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL,
		);
		
		if (!is_null($contactUsername = bgpc("contactUsername"))) {
			$validation_fields["contactUsername"] = INPUT_RESTRICTION_USERNAME|INPUT_RESTRICTION_REQUIRED;
			if (!ContactManager::isUsernameUnique($contactUsername, $contact->id)) {
				$this->invalidParams[] = "contactUsername-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
			}
		} else if (!$contact->isPrimary() && !is_null($contactEmail = bgpc("contactEmail"))) {
			if (!ContactManager::isUsernameUnique($contactEmail, $contact->id)) {
				$this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
			}
		}
		if (!is_null($contactEmail = bgpc("contactEmail")) && !ContactManager::isEmailUnique($contactEmail, $contact->id)) {
			$this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
		}

		if ("" != bgpc("contactPassword")) {
			$validation_fields["contactPassword"] = array(INPUT_RESTRICTION_PASSWORD|INPUT_RESTRICTION_REQUIRED, array("confirm" => "contactPasswordConfirm"));
			$contactFields[] = "contact_password";
		}
		
		$this->_validateZip();
        $this->_validateAndTransformCodeState();
        $this->validateMultiInputs($validation_fields, array(), array(), 1);

		try {
			foreach ( $contactFields as $attrName ) {
				if ( false === ($paramName = getDBXMLCol($attrName)) || is_null($paramValue = bgpc($paramName)) ) {
					continue;
				}
				if ("contact_password" == $attrName) {
					$contact->$attrName = \Billing\Manager\CryptManager::getPasswordHash($paramValue);
                    $contact->contact_password_type = \Billing\Manager\CryptManager::TYPE_CRYPT;
					$contact->plainPassword = $paramValue;
					continue;
				}
				$contact->$attrName = $paramValue;
			}
			$contact->emailActionIDs = (array)bgpc("emailActionIDs");
			if (!$contact->isPrimary() && is_null($contactUsername) && !is_null($contactEmail)) {
				$contact->contact_username = $contact->contact_email;
			}
			ContactManager::update($contact, $contact->plainPassword);
		} catch (ProductException $ex) {
			billing_put_error($ex->getMessage());
			$this->validationRedirect(array(), array(), true);
		}

		if ($contact->isPrimary()) {
			redirect("clients.php", getActionID("ShowClientDetails"), array("clientID" => $contact->client_id));
		} else {
			redirect("clients.php", getActionID("ShowClientContacts"), array("clientID" => $contact->client_id));
		}
	}

	protected function validationRedirect($extraArgs = array(), $referralArgs = array(), $force = false)
	{
		$this->setReferralActionFile("clients.php");
		$this->setReferralActionID(getActionID("EditClientContact"));
		$extraArgs = array_merge($extraArgs, array("clientID" => bgpc("clientID"), "clientContactID" => bgpc("clientContactID")));
		return parent::validationRedirect($extraArgs, $referralArgs, $force);
	}
}
