<?php

require_once("lib-action/action.php");

use Billing\Manager\CustomerManager;

class EditClientResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$customerFields = array(
			"client_company",
			"client_type",
			"skin_id",
			"client_status",
			"client_status_reason",
			"client_billing_day",
			"default_billing_account_id",
			"apply_tax",
			"tax_id_number",
		);

		$validation_fields = array(
			"clientCompany" => INPUT_RESTRICTION_STRING,
			"clientType" => INPUT_RESTRICTION_UNSIGNED_INT,
			"skinID" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"clientStatus" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"clientStatusReason" => INPUT_RESTRICTION_STRING,
			"clientBillingDay" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"defaultBillingAccountID" => INPUT_RESTRICTION_UNSIGNED_INT,
			"applyTax" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"taxIDNumber" => INPUT_RESTRICTION_STRING,
		);

		$this->validateMultiInputs($validation_fields, array(), array(), 1);

		$customer = Factory()->Customer(bgpc("clientID"));
		try {
			foreach ( $customerFields as $attrName ) {
				if ( false === ($paramName = getDBXMLCol($attrName)) || is_null($paramValue = bgpc($paramName)) ) {
					continue;
				}
				$customer->$attrName = $paramValue;
			}
			CustomerManager::update($customer);
		} catch (ProductException $ex) {
			billing_put_error($ex->getMessage());
			$this->validationRedirect(array(), array(), true);
		}
	
		redirect("clients.php", getActionID("ShowClientDetails"), array("clientID" => $customer->id));
	}

	protected function validationRedirect($extraArgs = array(), $referralArgs = array(), $force = false)
	{
		$this->setReferralActionFile("clients.php");
		$this->setReferralActionID(getActionID("EditClient"));
		$extraArgs = array_merge($extraArgs, array("clientID" => bgpc("clientID")));
		return parent::validationRedirect($extraArgs, $referralArgs, $force);
	}
}
