<?php

require_once("lib-action/action.php");
require_once("lib-tk/toolkit.php");

use Billing\Object\Subscription;
use Billing\Manager\ResellerManager;
use Billing\Manager\SubscriptionManager;
use Billing\TaskManager\Struct\Task;
use Billing\TaskManager\Client as TaskManagerClient;

/**
 * This action will take a client ID and change their status and their packages
 * based on the input.  It will not be used to activate new clients for now, difference
 * being, activation handles the queue differently.  Needs more thought...
 *
 * * Pass the clientID
 * _REQUEST["clientID"] = 4;
 *
 * _REQUEST["newStatusID"] = 3; 3=fraudulent
 * _REQUEST["purge"] = 1; 1 to delete this client, pass something for the newStatusID for code's sake
 *
 * * Reason String
 * _REQUEST["clientStatusReason"];  "This client was killing my server"
 *
 * _REQUEST["packages"][1] = array("suspend,suspend");  //Suspend package 1 and push a panel suspend call into the queue
 *                    [4] = array("suspend,suspend");
 *                    [7] = array("suspend,none");  //Suspend package 7 and don't do anything if panel tied
 *                    [8] = array("delete,none");  //Delete the package and don't do anything if panel tied
 *                   [11] = array("delete,suspend");  //Delete the package and suspend the panel account
 *                   [12] = array("unsuspend,unsuspend");  //Unsuspend the package and make the unsuspend panel call
 */
class ManageClientActionResponse extends MB_Action
{
	public $params = array("clientID");

	public function execute()
	{
		global $CONF;
		parent::execute();

		$this->setReferralActionID((int) $_REQUEST["referralActionID"]);
		if(!$_REQUEST["referralActionFile"]) {
			$this->setReferralActionFile("clients.php");
		} else {
			$this->setReferralActionFile($_REQUEST["referralActionFile"]);
		}

		if (!getCurrentSession()->isValidToken()) {
			billing_put_error(blmsg("TRANS_ERROR_TOKEN"));
			return $this->_completeAction();
		}

		//First flip the client's status, seems silly, but do this for purge so that the panel items get set
		//without a bunch of repeditive code
		$customerId = bgpc("clientID");
		$clientStatusReason = bgpc("clientStatusReason");
		if ($customerId && !is_null($newStatusID = bgpc("newStatusID")) && 0 <= $newStatusID) {
			//Flip the client status
			try {
				$customer = Factory()->Customer($customerId);
				if ($customer->isReseller()) {
					$reseller = $customer->getReseller();
					$reseller->status = $newStatusID;
					$reseller->status_reason = $clientStatusReason;
					ResellerManager::update($reseller);
				} else {
					$customer->client_status = $newStatusID;
					$customer->client_status_reason = $clientStatusReason;
					require_once("object_managers/ClientManager.php");
					ClientManager::update($customer);
				}
			} catch (ProductException $e) {
				billing_put_error($e->getMessage());
				return $this->_completeAction();
			}
			if ("" != $clientStatusReason) {
				updateFor("clients", array(
					"clientID" => $customerId,
					"clientNotes" => array(
						"clientNote" => array(
							"adminID" => $CONF["user"]["id"],
							"clientNoteDateCreated" => mktime(),
							"clientNoteText" => $clientStatusReason,
						)
					)
				));
			}
		}

		if(is_array($_REQUEST["packages"])) {
			$subscriptions = $_REQUEST["packages"];
			foreach($subscriptions as $subscriptionID => $newActions) {

				$newActions = explode(",",$newActions[0]);
				//Possible package status
				//	suspend, unsuspend, cancel, archive
				//
				//Possible panel actions
				//	suspend, unsuspend, delete
				$this->changeSubscriptionStatus($subscriptionID, $newActions);
			}

			// Now drop this client.  The code above should have taken care of their panel items
			if((int) $_REQUEST["purge"]) {
				deleteForKey("clientsGeneric", (int) $_REQUEST["clientID"]);
			}
		} // is packages an array

		return $this->_completeAction();
	}

   /**
    * This function will do the leg work.  Pass it the subscription ID and the array of
    * status fields as array(the new subscription status, what to do if tied to a panel)
    * EX: changeSubscriptionStatus(1, array("archive","delete"))
    *
    */
	private function changeSubscriptionStatus($subscriptionID, $action=array())
	{
		//Possible subscription status
		//	suspend, unsuspend, cancel
		//
		//Possible panel actions
		//	suspend, unsuspend, delete

		dbg("Passed in: ");
		dbg($action);

		$subscription = Factory()->Subscription($subscriptionID);

		$panelAction = NULL;
		switch ($action[1]) {
			case "suspend":
				$panelAction = SubscriptionManager::PANEL_ACTION_SUSPEND;
			break;
			case "delete":
				$panelAction = SubscriptionManager::PANEL_ACTION_DELETE;
			break;
		}

		$subscriptionStatus = NULL;
		switch ($action[0]) {
			case "unsuspend":
				SubscriptionManager::activate($subscription, bgpc("clientStatusReason"));
				break;
			case "suspend":
			case "suspended":
				SubscriptionManager::suspend($subscription, bgpc("clientStatusReason"), $panelAction);
				break;
			case "cancel":
				SubscriptionManager::cancel($subscription, bgpc("clientStatusReason"), $panelAction);
				break;
		}
	}

	private function _completeAction()
	{
		redirect($this->getReferralActionFile(), $this->getReferralActionID(), array(
			"clientID" => bgpc("clientID"),
			"n" => uniqid(0)
		));
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
