<?php

require_once("lib-action/action.php");

use Billing\Manager\ContactManager;
use Billing\Manager\CryptManager;

class ResetContactPasswordResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		try {
			// generate the new password
			$newPassword = getRandomPassword(12,1);

			// update password
			$contact = Factory()->Contact(bgpc("clientContactID"));
			if ("msp" == PACKAGE_TYPE) {
				$contact->contact_password = CryptManager::getPasswordHash($newPassword);
				$contact->plainPassword = $newPassword;
                $contact->contact_password_type = CryptManager::TYPE_CRYPT;
			} else {
				$contact->contact_temp_password = CryptManager::getPasswordHash($newPassword);
			}
			ContactManager::update($contact, $contact->plainPassword);

			// send email
			require_once("lib-mbapi/include/mail/emailTemplates/passwordNotificationEmail.php");
			$email = new PasswordNotificationEmail($newPassword, $contact->id);
			if (!$email->execute()) {
				throw new ProductException(ProductException::E_FAIL_OPERATION, "Failed to send email");
			}

			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_PASSWORD_RESET");

			if (bgpc("redirectActionID") && bgpc("redirectActionFile")) {
				$numMatches = preg_match("/[a-zA-Z0-9\-\_]*\.php/", bgpc("redirectActionFile"), $matches);
				if (1 === $numMatches && $matches[0]) {
					redirect($matches[0]."?action=".bgpc("redirectActionID")."&".bgpc("redirectArgs"));
				}
			}
			redirect("clients.php", "ShowClientContacts", array("clientContactID" => $contact->id));
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
		}

		return $this->hasExecutedSuccessfully();
	}
}