<?php

require_once("admin/include/lib-action/configs/EditEncryptionConfigsConfirm.php");

class EditEncryptionConfigsResponse extends EditEncryptionConfigsConfirm
{
	public function execute()
	{
		MB_Action::execute();

		if ( $this->noEncryptionChanged() ) {
			throw new Exception("Encryption parameters was not changed");
		}

		if (!getCurrentSession()->isValidToken()) {
			billing_put_error(blmsg("TRANS_ERROR_TOKEN"));
			redirect("configs.php", getActionID("EditEncryptionConfigs"));
		}

		try {
			$this->validateRequest();

			require_once("object_managers/CryptManager.php");
			if ( !CryptManager::isEncryptionEnabled() ) {
				$encryption_settings = array("encryption_agree" => bgpc("encryption_agree"));
				switch(bgpc("encryption_mode")) {
					case "lek" :
						$lek_pin = bgpc("lek_pin");
						$encryption_settings["lek_pin"] = $lek_pin;
						$encryption_settings["encryption_hash"] = CryptManager::generateEncryptionHash($lek_pin);
						break;
					case "passphrase" :
						$encryption_settings["passphrase"] = bgpc("pass_phrase");
						break;
				}
				CryptManager::update($encryption_settings);
				billing_put_info(blmsg("TRANS_ENCRYPTION_SETTINGS_SAVED_MSG"));
				$this->successRedirect();
			}

			CryptManager::update(array("encryption_agree" => bgpc("encryption_agree")));

			if ( !CryptManager::isEncryptionEnabled() ) {
				billing_put_info(blmsg("TRANS_ENCRYPTION_SETTINGS_SAVED_MSG"));
				$this->successRedirect();
			}

			$neededCypherType = bgpc("neededCypherType");
			$priorCypherType = bgpc("priorCypherType");
			if ( "" == $priorCypherType ) {
				$priorCypherType = bgpc("neededCypherType");
			}

			$params = array(
				"vendorID" => getCurrentSession()->getVendorId(),
				"from" => array(
					"type" => $priorCypherType,
				),
				"to" => array(
					"type" => $neededCypherType,
				),
			);

			if ( !CryptManager::isLekEnabled() ) {
				$params["from"]["passPhrase"] = bgpc("old_pass_phrase");
			}

			switch ( $encryption_mode = bgpc("encryption_mode") ) {
				case "lek":
					$encryption_hash = CryptManager::generateEncryptionHash($lek_pin = bgpc("lek_pin"));
					$params["to"]["encryption"] = array(
						"encryptionHash" => $encryption_hash,
						"lekPin" => $lek_pin,
					);
					break;
				case "passphrase":
					$params["to"]["passPhrase"] = bgpc("pass_phrase");
					break;
				default:
					throw new Exception("Invalid encryption mode: {$encryption_mode}");
			}
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$q = new MBQuery("ProcessEncryption", "", $params);
		$r =& $q->getResult();

		if ( $r->getSuccess() ) {
			if($configParamID = IDByName("configParams", "priorCypherType")) {
				updateForKey("configParams", $configParamID, array("configParamValue" => $neededCypherType));
			}
			billing_put_info(blmsg("TRANS_ENCRYPTION_SETTINGS_SAVED_MSG"));
			$this->successRedirect();
		} else {
			foreach ( $r->getErrors() as $error ) {
				$this->addError($error["title"], $error["message"]);
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function successRedirect()
	{
		$this->sessionRedirect(admin_action_url("configs", "system", "show"));
	}
}
