<?php

require_once("lib-action/action.php");

use Billing\Manager\AffiliateProgramManager;
use Billing\Manager\CustomerManager;
use Billing\Manager\InvoiceManager;
use Billing\Manager\ResellerManager;
use Billing\Manager\SubscriptionManager;
use Billing\Object\Customer;
use Billing\Object\Plan;
use Billing\Object\Reseller;
use Billing\Object\Subscription;

class ShowSystemOverview extends MB_Action
{
	public function execute()
	{
		parent::execute();

        $vendorId = getCurrentSession()->getVendorId();

		$customerStatusStat = CustomerManager::getStatusStatistics($vendorId);
		$this->smartyVars["customers"]["active"] = $customerStatusStat[Customer::STATUS_ACTIVE];
		$this->smartyVars["customers"]["pending"] = $customerStatusStat[Customer::STATUS_PENDING];

		$subscriptionTypeStat = SubscriptionManager::getTypeStatistics(array(Subscription::STATUS_ACTIVE));
		$this->smartyVars["subscriptions"]["active"] = array_sum($subscriptionTypeStat);
		$this->smartyVars["subscriptions"]["hosting"] = $subscriptionTypeStat[Plan::TYPE_HOSTING];
		$this->smartyVars["subscriptions"]["domain"] = $subscriptionTypeStat[Plan::TYPE_DOMAIN];
		$this->smartyVars["subscriptions"]["ssl"] = $subscriptionTypeStat[Plan::TYPE_SSL_CERT];
		$this->smartyVars["subscriptions"]["sslFactory"] = $subscriptionTypeStat[Plan::TYPE_CERT];

		$this->smartyVars["invoices"] = InvoiceManager::getDueStatistics();
		$this->smartyVars["invoices"]["due_date"] = BillingLocale::getDateShort(time());
		$this->smartyVars["invoices"]["overdue_date"] = BillingLocale::getDateShort(time() + 24*60*60);
		$this->smartyVars["invoices"]["due_min"] = BillingLocale::getDecimal(0.01);

        if (!$vendorId) {
            $resellerStatusStat = ResellerManager::getStatusStatistics($vendorId);
            $this->smartyVars["resellers"]["active"] = $resellerStatusStat[Reseller::STATUS_ACTIVE];
            $this->smartyVars["resellers"]["pending"] = $resellerStatusStat[Reseller::STATUS_PENDING];
            $this->smartyVars["subscriptions"]["reseller"] = $subscriptionTypeStat[Plan::TYPE_RESELLER];
        }

		$this->smartyVars["affiliatePrograms"] = AffiliateProgramManager::getStatistics($vendorId);

		$this->addTemplate("dashboard/ShowSystemOverview");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
