<?php

require_once("lib-action/action.php");

use Billing\Manager\PaymentGatewayManager;

class ProcessTransactionRefund extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$transactionIDs = array_unique((array)$_REQUEST["transactionIDs"]);
		$allInvoiceIDs = array_unique((array)$_REQUEST["invoiceID"]);

		// first lets test if we have a transactionID at all
		if ($transactionIDs[0]["transactionID"]){
			// lets get all our transactions
			$this->smartyVars["transactions"] = correctArray(grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetTransactions",
					"params" => array(
						"transactionIDs" => array(
							"transactionID" => $transactionIDs,
						),
					),
				),
			)));
			dbg($this->smartyVars["transactions"]);

			$this->smartyVars["invoiceID"] = (int)$_REQUEST["invoiceID"];

			// lets get any line item info that we need
			$lineItemIDs = array_unique((array)$_REQUEST["lineItemIDs"]);
			if ($lineItemIDs[0]){
				$successRel = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetTransactionSuccessRel",
						"params" => array(
							"lineItemID" => $lineItemIDs,
							"transactionID" => $transactionIDs,
						),
					),
				)));
				$lineItemInfo = correctArray(grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetLineItems",
						"params" => array(
							"lineItemID" => $lineItemIDs,
						),
					),
				)));
				$this->smartyVars["lineItemNames"] = array();
				foreach ((array)$lineItemInfo as $li){
					$this->smartyVars["lineItemNames"][$li["lineItemID"]] = "(".TRANS_ID.": ".$li["lineItemID"].") ".billing_safetyhtml($li["lineItemDescription"]);
				}
				$this->smartyVars["lineItemsForTransactions"] = array();
				$this->smartyVars["transactionInvoiceIDs"] = array();
				$this->smartyVars["transactionRefundTotals"] = array();
				foreach ((array)$successRel as $sr){
					$this->smartyVars["lineItemsForTransactions"][$sr["transactionID"]][] = $sr;
					$this->smartyVars["transactionRefundTotals"][$sr["transactionID"]] += $sr["lineItemPaymentAmount"];
				}
			}

			$this->smartyVars["invoiceInfo"] = correctArray(grabFirstResultFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetInvoices",
					"params" => array(
						"invoiceID" => $this->smartyVars["invoiceID"],
					),
				),
			)));

			foreach ((array)$this->smartyVars["transactions"] as $k=>$v) {
				$transaction = Factory()->PaymentTransaction($v["transactionID"]);
				$this->smartyVars["transactions"][$k]["gatewayActions"] = array("refund" => blmsg("TRANS_REFUND"));
				if ( "" != $v["paymentGatewayID"] ) {
					$gateway = Factory()->PaymentGateway($v["paymentGatewayID"]);
					if (PaymentGatewayManager::isOperationSupported($gateway, PaymentGatewayManager::OPERATION_PARTIAL_REFUND)) {
						if (count($this->smartyVars["lineItemsForTransactions"])){
							$this->smartyVars["transactions"][$k]["gatewayActions"]["partial_refund"] = blmsg("TRANS_PARTIAL_REFUND");
						}
						$this->smartyVars["transactions"][$k]["canPartial"] = 0; // disable partial refunds due to they are not implemented correctly
					}
					if (0.0 == $transaction->transaction_amount_refunded &&
						PaymentGatewayManager::isOperationSupported($gateway, PaymentGatewayManager::OPERATION_VOID)
					) {
						$this->smartyVars["transactions"][$k]["gatewayActions"]["void"] = blmsg("TRANS_VOID");
					}
				} else {
					show($v);
				}

				if ($lineItemIDs[0]) {
					$this->smartyVars["transactions"][$k]["selectedAction"] = 'partial_refund';
				} else {
					$day = date("z", $v["transactionDateCreated"]);
					$today = date("z");
					$this->smartyVars["transactions"][$k]["selectedAction"] =  ($day != $today) ? 'refund' : 'void';
				}

				$this->smartyVars["transactions"][$k]['billingAccount'] = oneByID(':billingAccounts', $this->smartyVars["transactions"][$k]['billingAccountID']);
				if (!$this->smartyVars["transactions"][$k]['billingAccount']['billingAccountID']){
					$this->smartyVars["transactions"][$k]['noBillingAccount'] = 1;
				}
			}

			$this->addTemplate("invoices/ProcessTransactionRefund", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
