<?php

require_once("lib-action/action.php");
require_once("lib-billing/include/accounting/Refund.php");

use Billing\Manager\CryptManager;
use Billing\Manager\PaymentManager;
use Billing\Object\LedgerAccount;

class ProcessTransactionRefundResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (is_null($transactionID = bgpc("transactionID"))) {
			throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, "Transaction not found: id={$transactionID}");
		}
		if (!getCurrentSession()->isValidToken()) {
			$this->_redirectWithErrorMessage(blmsg("TRANS_ERROR_TOKEN"));
		}
		if ("" == ($key = bgpc("key"))) {
			$this->_redirectWithErrorMessage("No encryption key given.");
		} else if (CryptManager::isEncryptionEnabled()) {
			try {
				CryptManager::checkEncryptionKey($key, getCurrentSession()->user_id);
			} catch(ProductException $e) {
				$this->_redirectWithErrorMessage($e->getMessage());
			}
		}

		$invoiceID = ((int) $_REQUEST["invoiceID"]);
		$markUnpaid = (int)$_REQUEST["markUnpaid"];

		/* disable partial refunds due to they are not implemented correctly
		if ($_REQUEST["refundType"]=="partial"){
			$_REQUEST["refundAmount"] = BillingLocale::getNumberFromInput($_REQUEST["lesserAmount"]);
		}
		*/

		CryptManager::setKey($key);

        $gatewayAction = "void" == bgpc("gatewayAction") ? "void" : "refund";

		$transaction = Factory()->Transaction($transactionID);

		if (is_null($billingAccount = $transaction->ClientBillingAccount)) {
			$this->_redirectWithErrorMessage("Sorry, the billing account this refund is for was deleted.  To perform this refund please go to your merchant account provider.");
		}

		$transactionAmountRemained = round(doubleval($transaction->transaction_amount) - doubleval($transaction->transaction_amount_refunded), 2);
		$refundAmount = doubleval(bgpc("refundAmount")) ? doubleval(bgpc("refundAmount")) : $transactionAmountRemained;
		if ($refundAmount > $transactionAmountRemained) {
			$refundAmount = $transactionAmountRemained;
		}

		try {
			if ("void" == bgpc("gatewayAction")) {
				$success = PaymentManager::void($transaction, $refundAmount);
			} else {
				$success = PaymentManager::refund($transaction, $refundAmount);
			}
			if (!$success) {
				$this->_redirectWithErrorMessage($transaction->transaction_notes);
			}
		} catch (ProductException $e) {
			$this->_redirectWithErrorMessage($e->getMessage());
		}

		$invoiceInfo = correctArray(grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetInvoices",
				"params" => array(
					"invoiceID" => $invoiceID,
					"getInvoiceLineItemsData" => 1,
				),
			),
		)));

		$lineItemIDs = (array)$_REQUEST["lineItemIDs"];

		if (array_sum($lineItemIDs)){

			if ($markUnpaid){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetInvoices",
						"subCommand" => "update",
						"params" => array(
							"invoiceID" => $invoiceID,
							"invoiceStatus" => getAPIStatusID("invoice", "due"),
							"invoiceAmountPaid" => $invoiceInfo["invoiceAmountPaid"]-$refundAmount,
						),
					),
				)), 1);

				foreach ((array)$lineItemIDs as $liID){
					dispatchMBAPI(toXML(array(
						"mbapi" => array(
							"command" => "SetTransactionSuccessRel",
							"subCommand" => "delete",
							"params" => array(
								"lineItemID" => $liID,
								"transactionID" => $transactionID,
							),
						),
					)), 1);
				}
			}

			$return = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
						"refundAmount" => $refundAmount,
						"ledgerAccountName" => $markUnpaid ? LedgerAccount::NAME_ACCOUNTS_RECEIVABLE : LedgerAccount::NAME_REFUND_EXPENSE,
						"invoiceID" => $invoiceID,
					),
				),
			)), 1);
			if (!$return["mabpi"][0]["header"][0]["success"][0]){
				dbg($return);
			}

		} else {

			$clientID = $invoiceData["lineItems"]["lineItem"][0]["clientID"];

			if ($markUnpaid){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetTransactionSuccessRel",
						"subCommand" => "delete",
						"params" => array(
							"transactionID" => $transactionID,
						),
					),
				)));
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetInvoices",
						"subCommand" => "update",
						"params" => array(
							"invoiceID" => $invoiceID,
							"invoiceStatus" => getAPIStatusID("invoice", "due"),
							"invoiceAmountPaid" => $invoiceInfo["invoiceAmountPaid"] - $refundAmount,
						),
					),
				)));
			}

			$merge = array();
			if ($refundAmount != $transaction->transaction_amount) {
				$merge = array(
					"refundAmount" => $refundAmount,
				);
			}

			$return = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
						"ledgerAccountName" => $markUnpaid ? LedgerAccount::NAME_ACCOUNTS_RECEIVABLE : LedgerAccount::NAME_REFUND_EXPENSE,
						"invoiceID" => $invoiceID,
					) + $merge,
				),
			)), 1);
			if (!$return["mabpi"][0]["header"][0]["success"][0]){
				dbg($return);
			}

		}

		if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
			redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
		} else {
			redirect("invoices.php?action=".getActionID("ShowInvoices")."&sid=".secureInput($_REQUEST["sid"]));
		}
	}

	private function _redirectWithErrorMessage($message)
	{
		billing_put_error($message);
		redirect("invoices.php", getActionID("ProcessTransactionRefund"), array(
			"invoiceID" => bgpc("invoiceID"),
			"transactionIDs" => array(bgpc("transactionID")),
			"lineItemIDs" => bgpc("lineItemIDs", array()),
		));
	}
}