<?php

require_once("admin/include/lib-action/packages/AddHistoricalPackage.php");
require_once("lib-mbapi/include/modules/gateway/gateway.php");

use Billing\Manager\CryptManager;
use Billing\Object\PanelGroup;
use Billing\Object\Plan;

class AddHistoricalPackageResponse extends AddHistoricalPackage
{
	public function execute()
	{
		global $CONF;

		if (!getCurrentSession()->isValidToken()) {
			$this->_redirectWithError(blmsg("TRANS_ERROR_TOKEN"));
		}

		do {
			$clientID = getParam("clientID");
			$orderFormID = getParam("orderFormID");
			$productID = getParam("productID");
			$productVariantID = getParam("productVariantID");
			$cycleID = getParam("cycleID");

			// check vendor for input data
			$client = Factory()->Customer($clientID);
			$store = Factory()->Store($orderFormID);
			$product = Factory()->Plan($productID);
			$vendorId = getCurrentSession()->getVendorId();
			if (!$client->checkVendorId($vendorId) || !$store->checkVendorId($vendorId) || !$product->checkVendorId($vendorId)) {
				$this->_redirectWithError('TRANS_ERROR_DATA_INTEGRITY');
			}

			$this->assignMode();
			$this->assignProductInfo();
			$this->assignClientInfo();

			$packageDomainID = bgpc("packageDomainID");
			$billingAccountID = $_REQUEST["billingAccountID"];

			if($this->product["productStopNumCycles"] != 1) {
				$packageNextRenewal = $this->handleDateSelector("packageNextRenewal", mktime());
			}
			$packageStartDate = $this->handleDateSelector("packageStartDate", mktime());

			if($this->type == "domain") {
				$registerDomain = $this->handleDateSelector("registerDomain", false, !$this->historical);
			} else {
				$serverGroupData = array();
				foreach($this->serverGroups as $serverGroupID => $serverGroupName) {
					$key = "sg" . $serverGroupID;
					$serverGroupData[$serverGroupID] = array();
					$serverGroupData[$serverGroupID]["configGroupID"] = bgpc($key . "panel");
					if(1 == bgpc("sg{$serverGroupID}create")) {
						if(isset($_REQUEST[$key . "username"])) {
							$serverGroupData[$serverGroupID]["username"] = bgpc($key . "username");
							$serverGroupData[$serverGroupID]["password"] = bgpc($key . "password");
						}
						$serverGroupData[$serverGroupID]["provision"] = $this->handleDateSelector($key . "create", false);
					} else {
						$serverGroupData[$serverGroupID]["provision"] = false;
					}
				}
			}
			$generalPrice = BillingLocale::getNumberFromInput($_REQUEST["generalPrice"]);
			$setupPrice = BillingLocale::getNumberFromInput($_REQUEST["setupPrice"]);

			$overrideDateDue = $this->handleDateSelector("overrideDateDue", 0);

			$query = array();
			if (!$_REQUEST["emailOptions"]){
				$emailOptions = array(
					"enableInvoiceEmail" => 0,
					"disableNewOrderEmail" => 1,
				);
			} elseif ((int)$_REQUEST["emailOptions"]==1){
				$emailOptions = array(
					"enableInvoiceEmail" => 0,
					"disableNewOrderEmail" => 0,
				);
			} elseif ((int)$_REQUEST["emailOptions"]==2){
				$emailOptions = array(
					"enableInvoiceEmail" => 1,
					"disableNewOrderEmail" => 0,
				);
			}

            $enableCharging = bgpc("enableCharging");
            if (1 == $enableCharging) {
                $store = Factory()->OrderForm($orderFormID);
                $billingAccountType = ("" != $billingAccountID) ? Factory()->ClientBillingAccount($billingAccountID)->billing_account_type : null;
                switch ($billingAccountType) {
				case ClientBillingAccount::TYPE_CREDIT_CARD:
					if (is_null($store->PrimaryGateway) || $store->PrimaryGateway->hasCapability(GATEWAY_3D_SECURE)) {
						$enableCharging = 0;
					}
					break;
				case ClientBillingAccount::TYPE_HSPC_BANK_TRANSFER:
					if (is_null($store->HSPCBankTransferGateway)) {
						$enableCharging = 0;
					}
					break;
				default:
					$enableCharging = 0;
                }
				if (0 == $enableCharging && 1 == bgpc("generateInvoice")) {
					billing_put_warning(blmsg("TRANS_CHARGE_IMMEDIATELY_NOT_POSSIBLE_MSG"));
				}
            }
			$query += array(
				"orderFormID" => $orderFormID,
				"paymentType" => 9999,
				"disableInvoicing" => empty($_REQUEST["generateInvoice"]) ? 1 : 0,
				"overrideDateDue" => $overrideDateDue,
				"disableCharging" => (int) (bool) !$enableCharging,
				"disableFraudguardian" => 1,
				"disableClientUpdates" => 1,
				"client" => array(
					"clientID" => $clientID,
				),
			);
			$query += $emailOptions;

			$product = array(
				"ID" => 14736251,
				"productID" => $productID,
				"productVariantID" => $productVariantID,
				"cycleID" => $cycleID,
				"quantity" => 1,
				"taxZoneGroupID" => $this->productVariant["taxZoneGroupID"],
				"comments" => ($_REQUEST["comments"])?secureInput($_REQUEST["comments"]):TRANS_MANUALLY_CREATED_BY_ADMIN.": " . $CONF["user"]["adminData"]["adminUsername"][0],
				"price" => (float) $generalPrice,
				"setup" => (float) $setupPrice,
				"packageDateNextRenewal" => $packageNextRenewal ? $packageNextRenewal : $packageStartDate,
				"packageDateBeginRenewal" => $packageStartDate,
			);

			$servers = array();
			foreach((array)$serverGroupData as $serverGroupID => $serverData) {
				$server = array();

				$server["serverGroupID"] = $serverGroupID;
				if($configGroupID = $serverData["configGroupID"]) $server["configGroupID"] = $configGroupID;
				$server["packageUsername"] = $serverData["username"];
				//Adding this to figure out if this package should just be tied to the server -AC 11-9-07
				$server["serverTieOnly"] = ((int)$serverData["provision"])? 0 : 1;

				$servers[] = $server;
			}
			if($servers) $product["servers"] = array("server" => $servers);

			if($billingAccountID) $query["clientBillingAccount"] = array("billingAccountID" => $billingAccountID);

			$contact = $this->primaryContact;
			$TLD = $this->TLD;
			switch ($this->type) {
				case "domain":
					if($registerDomain) {
						$registrar = Factory()->ProductVariant($productVariantID)->TLD->BillingRegistrar;
						if ("" != $packageDomainID) {
							$packageDomain = Factory()->PackageDomain($packageDomainID);
							$domainSLD = $packageDomain->domain_sld;
							$domainTLD = $packageDomain->domain_tld;
						} else {
							$domainSLD = bgpc("domainName");
							$domainTLD = $TLD["tldExtension"];
                            if (false !== strpos($domainSLD, '.')) {
                                $this->domainError(blmsg("TRANS_INCORRECT_DOMAIN_NAME_INCLUDE_DOT", array("DOMAIN_NAME" => billing_safetyhtml("{$domainSLD}.{$domainTLD}"))), $domainSLD, "");
                            }
							$this->validateDomainName($domainSLD, $domainTLD);
							$this->checkDomainAvailability($registrar, $domainSLD, $domainTLD);
						}

						if (!isset($_REQUEST["registrantInfo"])) {
							redirect("packages.php", getActionID("EditDomainRegistrationInfo"), array(), array(
								"addHistoricalPackageResponseRequest" => bgpc_array($_REQUEST),
								"clientID" => bgpc("clientID"),
								"registrarID" => $registrar->id,
								"domainSLD" => $domainSLD,
								"domainTLD" => $domainTLD,
							));
						}

						$query["provisionDateRule"] = array(
							"domainID" => (int) $_REQUEST["packageDomainID"],
							"provisionDate" => $registerDomain,
						);

						if($_REQUEST["packageDomainID"]) {
							$product["domain"] = array(
								"domainID" => (int) $_REQUEST["packageDomainID"],
								"domainType" => getAPITypeID("shoppingdomains", "existing"),
							);
						} else if($_REQUEST["domainName"] && $TLD["tldExtension"]) {
							$product["domain"] = array(
								"domainID" => 14736251,
								"domainType" => getAPITypeID("shoppingdomains", "new"),
								"domainSLD" => $_REQUEST["domainName"],
								"domainTLD" => $TLD["tldExtension"],
							);
						}
						if (isset($_REQUEST["registrantInfo"])) {
							$product["domain"]["extraInfo"] = $_REQUEST["registrantInfo"];
						} else {
							$product["domain"]["extraInfo"] = array(
								"registrantFirstName" => $contact["contactFirstName"],
								"registrantLastName" => $contact["contactLastName"],
								"registrantAddress1" => $contact["contactAddress1"],
								"registrantAddress2" => $contact["contactAddress2"],
								"registrantCity" => $contact["contactCity"],
								"registrantStateProvince" => $contact["contactState"],
								"registrantPostalCode" => $contact["contactZip"],
								"registrantOrganizationName" => $contact["clientCompany"],
								"registrantJobTitle" => "",
								"registrantEmail" => $contact["contactEmail"],
								"registrantPhone" => $contact["contactPhone1"],
								"registrantFax" => $contact["contactFax"],
								"registrantCountry" => $contact["countriesID"],
							);
						}
					} else {
						if($_REQUEST["packageDomainID"]) {
							$query["provisionRules"] = array(
								"disableDomainIDs" => array("domainID" => (int) $_REQUEST["packageDomainID"]),
							);
							$product["domain"] = array(
								"domainID" => (int)$_REQUEST["packageDomainID"],
								"domainType" => getAPITypeID("shoppingdomains", "existing"),
							);
						} else if($_REQUEST["domainName"] && ($TLD["tldExtension"] || $_REQUEST["domainTLD"])) {
							$query["provisionRules"] = array(
								"disableDomainIDs" => array("domainID" => 14736251),
							);
							$product["domain"] = array(
								"domainID" => 14736251,
								"domainType" => getAPITypeID("shoppingdomains", "new"),
								"domainSLD" => $_REQUEST["domainName"],
								"domainTLD" => ($TLD["tldExtension"])?$TLD["tldExtension"]:$_REQUEST["domainTLD"],
							);
						}
					}
					break;
				case "hosting":
                    $domainSLD = bgpc("domainName", "");
                    $domainTLD =  bgpc("domainTLD", "");
					if($_REQUEST["packageDomainID"]) {
						$query["provisionRules"] = array(
							"disableDomainIDs" => array("domainID" => (int) $_REQUEST["packageDomainID"]),
						);
						$product["domain"] = array(
							"domainID" => (int)$_REQUEST["packageDomainID"],
							"domainType" => getAPITypeID("shoppingdomains", "existing"),
						);
					} else if("" != $domainSLD && "" != $domainTLD) {
                        $this->validateDomainName($domainSLD, $domainTLD);
						$query["provisionRules"] = array(
							"disableDomainIDs" => array("domainID" => 14736251),
						);
						$product["domain"] = array(
							"domainID" => 14736251,
							"domainType" => getAPITypeID("shoppingdomains", "manual"),
							"domainSLD" => $_REQUEST["domainName"],
							"domainTLD" => $_REQUEST["domainTLD"],
						);
					} else if(Plan::DOMAIN_REQUIRED == $this->domainRequired) {
                        $this->validateDomainName($domainSLD, $domainTLD);
					}
					// fall through
				default:
					$provisionDateRules = array();
					$disableProvisionIDs = array(
						"serverGroupID" => array(),
					);
					foreach($serverGroupData as $serverGroupID => $serverGroupAttrs) {
						if($serverGroupAttrs["provision"]) {
							$provisionDateRules[] = array(
								"serverGroupID" => $serverGroupID,
								"provisionDate" => $serverGroupAttrs["provision"],
							);
						} else {
							$disableProvisionIDs["serverGroupID"][] = $serverGroupID;
						}
					}
					$query["provisionRules"] = array(
						"provisionDateRule" => $provisionDateRules,
						"disableProvisionIDs" => $disableProvisionIDs,
					);
					break;
			}
			$query["products"] = $product;

			if (1 == $enableCharging && !is_null($encKey = bgpc("encKey"))) {
				if(CryptManager::MODE_PASSPHRASE == CryptManager::getEncryptionMode()) {
					try {
						CryptManager::checkEncryptionKey($encKey, getCurrentSession()->user_id);
					} catch(ProductException $ex) {
						$this->_redirectWithError($ex->getMessage());
					}
				}
				CryptManager::setKey($query["encKey"] = $encKey);
			}

            $coupon = reset(Factory()->Product($productID)->preappliedCouponList());
            if ($coupon && $coupon->canApply($clientID)) {
                $query["couponID"] = $coupon->id;
            }

            try {
                $q = new MBQuery("ProcessCheckout", "process", $query);
                dbg($q->getXMLQuery());
                if(!$this->addErrorsFromResult($r =& $q->getResult())) break;
                $rs =& $r->getSubjectFrom(array("mbapi", "results", "packageID", 0));
                $packageID = $rs->getData();
                updateForKey(":packages", $packageID, array(
                    "packageDateBeginRenewal" => $packageStartDate,
                ));
                if (isset($query["couponID"]) && $coupon) {
                    require_once('object_managers/CouponManager.php');
                    $coupon->coupon_count += 1;
                    CouponManager::update($coupon);
                }
            } catch (ProductException $e) {
                $this->_redirectWithError(blmsg($e->getMessage()));
            }

			if(!$this->goBack()) redirect("packages.php", getActionID("ShowPackageDetails"), array("packageID" => $packageID));
		} while(0);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function validateDomainName($domainSLD, $domainTLD)
	{
        if (!Billing_Checker::idnDomainName("{$domainSLD}.{$domainTLD}")) {
			$this->domainError(blmsg("TRANS_INCORRECT_DOMAIN_NAME", array("DOMAIN_NAME" => billing_safetyhtml("{$domainSLD}.{$domainTLD}"))), $domainSLD, $domainTLD);
		}
	}

	private function checkDomainAvailability($registrar, $domainSLD, $domainTLD)
	{
		try {
			require_once("object_managers/BillingRegistrarManager.php");
			$domain = reset(BillingRegistrarManager::checkDomainAvailability($registrar, array(array("sld" => $domainSLD, "tld" => $domainTLD))));
			if (BillingRegistrar::DOMAIN_AVAIL_STATUS_NOT_AVAIL == $domain["status"]) {
				$this->domainError(blmsg("TRANS_REGISTRAR_DOMAIN_TAKEN_MSG", array("DOMAIN_NAME" => billing_safetyhtml("{$domainSLD}.{$domainTLD}"))), $domainSLD, $domainTLD);
			} else if (BillingRegistrar::DOMAIN_AVAIL_STATUS_CANT_CHECK == $domain["status"]) {
				$this->domainError(
					blmsg(
						"TRANS_REGISTRAR_DOMAIN_CANT_CHECK_AVAILABILITY_MSG",
						array("DOMAIN_NAME" => billing_safetyhtml("{$domainSLD}.{$domainTLD}"), "TLD" => billing_safetyhtml($domainTLD), "NAME" => $registrar->name)
					),
					$domainSLD,
					$domainTLD
				);
			}
		} catch (ProductException $e) {
			$this->domainError(blmsg("TRANS_REGISTRAR_DOMAIN_CHECK_AVAILABILITY_FAILED_MSG", array(
				"DOMAIN_NAME" => billing_safetyhtml("{$domainSLD}.{$domainTLD}"),
				"ERROR" => billing_safetyhtml($e->getMessage()),
			)));
		}
	}

	private function domainError($message, $domainSLD = "", $domainTLD = "")
	{
        $this->_redirectWithError($message, array(
			"domainName" => $domainSLD,
			"domainTLD" => $domainTLD,
		));
	}

    private function _redirectWithError($message, $params = array())
    {
		billing_put_error($message);
        $params = array_merge($params, array(
			"mode" => bgpc("mode"),
			"clientID" => bgpc("clientID"),
			"orderFormID" => bgpc("orderFormID"),
			"productID" => bgpc("productID"),
			"productVariantID" => bgpc("productVariantID"),
			"cycleID" => bgpc("cycleID"),
            "billingAccountID" => bgpc("billingAccountID"),
		));
        foreach(Factory()->PanelGroupList() as $panelGroup) {
            $key = "sg{$panelGroup->id}create";
            if(!is_null(bgpc($key))) {
                $params[$key] = bgpc($key);
            }
        }
        $params["validationError"] = 1;
        redirect("packages.php", "AddHistoricalPackage", $params);
    }
}
