<?php
require_once ("lib-action/action.php");

use Billing\Database\Transaction;
use Billing\Manager\SubscriptionManager;
use Billing\Object\Subscription;
use Billing\Object\Plan;
use Billing\TaskManager\Struct\Task;
use Billing\TaskManager\Client as TaskManagerClient;

class EditPackageResponse extends MB_Action
{
	public function __construct()
    {
		parent::__construct();
		$this->params = array(
			"packageID",
			"clientContactID",
			"packageName",
			"packageQuantity",
			"packageDateNextRenewal",
			"packageDateStopRenewal",
			"packageActive",
			"packageStatus",
			"packageComments",
			"packageType",
			"packageCancellationType",
			"cycleID",
			"usageDateNextRenewal",
			"usageCycleID",
			"usageCalculateOnly",
			"packageClientPermissions",
		);
		$this->setupDateRequestArgs();
		if ($_REQUEST["packageDateNextRenewalYear"])$this->reconstructDateRequest(array("packageDateNextRenewal"));
		if ($_REQUEST["usageDateNextRenewalYear"])$this->reconstructDateRequest(array("usageDateNextRenewal"));
		if ($_REQUEST["packageDateStopRenewalYear"])$this->reconstructDateRequest(array("packageDateStopRenewal"));
	}

	public function execute()
    {
		parent::execute();
		$this->setReferralActionID(getActionID("EditPackage"));
		$this->setReferralActionFile("packages.php");

		do {
			if($this->err($packageID = $this->validateParam("packageID", INPUT_RESTRICTION_POSITIVE_INT|INPUT_RESTRICTION_REQUIRED), "No appropriate package ID given")) break;
			$this->_subscription = Factory()->Subscription($packageID);

			if (!getCurrentSession()->isValidToken()) {
				$this->_displayError(blmsg('TRANS_ERROR_TOKEN'));
			}

			// if we are doing a server association then lets do that then redirect
			if ($_REQUEST["task"]=="serverassociate"){
				list($serverGroupID, $configGroupID) = explode("|", $_REQUEST["serverID"]);
				// get the attributes if they are there already
				$attributes = arrayFor(":packageAttributes", $x = array(
					"packageID" => $packageID,
					"comparisonOperator" => "=",
					"packageAttributeName" => array(
						"panel_status",
						"domainUserName",
					),
					"serverGroupID" => $serverGroupID,
				));
				$keyedAttributes = array();
				foreach ($attributes as $attr){
					$keyedAttributes[$attr["packageAttributeName"]] = $attr;
				}

				if ($keyedAttributes["panel_status"]){
					updateForKey(":packageAttributes", $keyedAttributes["panel_status"]["packageAttributeID"], array(
						"configGroupID" => $configGroupID,
					));
				} else {
					insertFor(":packageAttributes", array(
						"packageID" => $packageID,
						"serverGroupID" => $serverGroupID,
						"configGroupID" => $configGroupID,
						"packageAttributeName" => "panel_status",
						"packageAttributeValue" => "none",
						"packageAttributeDateCreated" => time(),
					));
				}

				if ($_REQUEST["domainUserName"]){
					if ($keyedAttributes["domainUserName"]){
						updateForKey(":packageAttributes", $keyedAttributes["domainUserName"]["packageAttributeID"], array(
							"configGroupID" => $configGroupID,
							"packageAttributeValue" => $_REQUEST["domainUserName"],
						));
					} else {
						insertFor(":packageAttributes", array(
							"packageID" => $packageID,
							"serverGroupID" => $serverGroupID,
							"configGroupID" => $configGroupID,
							"packageAttributeName" => "domainUserName",
							"packageAttributeValue" => $_REQUEST["domainUserName"],
							"packageAttributeDateCreated" => time(),
						));
					}
				}

				$ids = IDsOf("packageAttributes", array(
					"packageID" => $packageID,
					"serverGroupID" => $serverGroupID,
				));
				updateFor("packageAttributes", array(
					"packageAttributeID" => $ids,
					"configGroupID" => $configGroupID,
					"serverGroupID" => $serverGroupID,
				));

				redirect("packages.php", "ShowPackageDetails", array(
					"packageID" => $packageID,
				));
				break;
			} elseif (bgpc("task")=="panelAction") {
				if ($panelAction = bgpc("panelAction")){
					$panelQuery = new MBQuery(
						"ProcessPanel", $panelAction,
						array(
							'subCommand' => $panelAction,
							'serverGroupID' => (int)bgpc("serverGroupID"),
							'configGroupID' => (int)bgpc("configGroupID"),
							'packageID' => $packageID,
						)
					);
					$task = new Task(
						"ProcessPanel ({$panelAction})",
						$panelQuery->getXMLQuery(),
						array(
							"panelGroupId" => (int)bgpc("serverGroupID"),
							"panelId" => (int)bgpc("configGroupID"),
							"subscriptionId" => $packageID,
						)
					);
					TaskManagerClient::scheduleMbapi($task, Factory()->Subscription($packageID)->vendor_id);
				}
				redirect("packages.php", "ShowPackageDetails", array(
					"packageID" => $packageID,
				));
				break;
			} elseif(!is_null($subscriptionAction = bgpc("subscriptionAction"))) {
				$subscription = Factory()->Subscription($packageID);
				try {
					switch($subscriptionAction) {
					case "suspend" :
						SubscriptionManager::suspend($subscription, blmsg("TRANS_SUBSCRIPTION_SUSPEND_REASON_MANUAL"));
						billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_SUSPEND_SUCCESS_MSG'));
						break;
					case "activate" :
						SubscriptionManager::activate($subscription);
						if ($subscription->isNotProvisionedType()) {
							billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_ACTIVATE_WITHOUT_PROVISIONING_SUCCESS_MSG'));
						} else {
							billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_ACTIVATE_SUCCESS_MSG'));
						}
						break;
					case "cancel" :
						$subscription->package_pending_cancellation = Subscription::PENDING_CANCELLATION_TYPE_ADMIN;
						SubscriptionManager::update($subscription);
						billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_CANCEL_SUCCESS_MSG',
							array("DATE" => billing_get_user_locale()->getLocaleDate($subscription->package_date_next_renewal))
						));
						break;
					case "withdrawCancellation" :
						$subscription->package_pending_cancellation = Subscription::PENDING_CANCELLATION_TYPE_NO;
						SubscriptionManager::update($subscription);
						billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_WITHDRAW_CANCELLATION_SUCCESS_MSG',
							array("DATE" => billing_get_user_locale()->getLocaleDate($subscription->package_date_next_renewal))
						));
						break;
					case "cancelImmediately" :
						SubscriptionManager::cancel($subscription);
						billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_CANCEL_IMMEDIATELY_SUCCESS_MSG'));
						break;
					case "cancelUpgrade" :
						SubscriptionManager::cancelUpgrade($subscription);
						billing_put_info(blmsg('TRANS_SUBSCRIPTION_OPERATION_CANCEL_UPGRADE_DOWNGRADE_SUCCESS_MSG'));
						break;
					}
				} catch (ProductException $e) {
					billing_put_error($e->getMessage());
				}
				redirect("packages.php", "ShowPackageDetails", array("packageID" => $packageID));
				break;
			}

			$configGroupID = $this->validateParam("configGroupID", INPUT_RESTRICTION_POSITIVE_INT|INPUT_RESTRICTION_REQUIRED);
			if($this->err($configGroupID)) {


				$packageQuery = new MBQuery(
					"GetPackages", "",
					array(
						"packageID" => $packageID,
						"getPackageAttributeData" => 1,
						"getPackageAddonData" => 1,
						"getDomainData" => 1,
					)
				);
				if(!$this->addErrorsFromResult($packageResult =& $packageQuery->getResult())) break;

				// base editing
				$this->invalidParams = array();

				$params = array();
				dbg("before it");
				dbg($_REQUEST);
				if (isset($_REQUEST["packageActive"]))$params["packageActive"] = $this->validateParam("packageActive", INPUT_RESTRICTION_BOOL);
				if (isset($_REQUEST["cycleID"]))$params["cycleID"] = $this->validateParam("cycleID", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["clientContactID"]))$params["clientContactID"] = $this->validateParam("clientContactID", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["packageStatus"]))$params["packageStatus"] = $this->validateParam("packageStatus", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["packageStatusReason"]))$params["packageStatusReason"] = $this->validateParam("packageStatusReason", INPUT_RESTRICTION_STRING);
				if (isset($_REQUEST["packageCustomerStatusReason"]))$params["packageCustomerStatusReason"] = $this->validateParam("packageCustomerStatusReason", INPUT_RESTRICTION_STRING);
				if (isset($_REQUEST["packageName"]))$params["packageName"] = $this->validateParam("packageName", INPUT_RESTRICTION_STRING);
				if (isset($_REQUEST["packageQuantity"]))$params["packageQuantity"] = $this->validateParam("packageQuantity", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["packageComments"]))$params["packageComments"] = $this->validateParam("packageComments", INPUT_RESTRICTION_STRING);
				if (isset($_REQUEST["packageType"]))$params["packageType"] = $this->validateParam("packageType", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["packageDateNextRenewal"]))$params["packageDateNextRenewal"] = $this->validateParam("packageDateNextRenewal", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["packageDateStopRenewalChecked"]) && (int)$_REQUEST["packageDateStopRenewalChecked"]){
					if (isset($_REQUEST["packageDateStopRenewal"]))$params["packageDateStopRenewal"] = $this->validateParam("packageDateStopRenewal", INPUT_RESTRICTION_INT);
				} elseif (isset($_REQUEST["packageDateStopRenewalChecked"]) && !(int)$_REQUEST["packageDateStopRenewalChecked"]){
					$params["packageDateStopRenewal"] = 0;
				}
				if (isset($_REQUEST["usageDateNextRenewalChecked"]) && (int)$_REQUEST["usageDateNextRenewalChecked"]){
					if (isset($_REQUEST["usageDateNextRenewal"]))$params["usageDateNextRenewal"] = $this->validateParam("usageDateNextRenewal", INPUT_RESTRICTION_INT);
				} elseif (isset($_REQUEST["usageDateNextRenewalChecked"]) && !(int)$_REQUEST["usageDateNextRenewalChecked"]){
					$params["usageDateNextRenewal"] = 0;
				}
				if (isset($_REQUEST["usageCycleID"]))$params["usageCycleID"] = $this->validateParam("usageCycleID", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["usageCalculateOnly"]))$params["usageCalculateOnly"] = $this->validateParam("usageCalculateOnly", INPUT_RESTRICTION_BOOL);
				if (isset($_REQUEST["packageClientPermissions"]))$params["packageClientPermissions"] = $this->validateParam("packageClientPermissions", INPUT_RESTRICTION_INT);
				if (isset($_REQUEST["packageCancellationType"]))$params["packageCancellationType"] = $this->validateParam("packageCancellationType", INPUT_RESTRICTION_INT);

				if (count($this->invalidParams)){
					dbg("Doing validation redirect");
					$this->validationRedirect(array("packageID" => (int)$_REQUEST["packageID"]));
					break;
				} else {
					dbg('Update subscription with id = ' . $packageID);
					dbg($params);

                    $this->_changePrimaryDomain();

					if (isset($params["packageActive"])) {
						$this->_subscription->package_active = $params["packageActive"];
					}
					if (isset($params["packageStatus"])) {
						$this->_subscription->package_status = $params["packageStatus"];
					}
					SubscriptionManager::update($this->_subscription);

					// update generic properties of subscription
					$params["packageID"] = $packageID;
					updateFor("packagesGeneric", $params);
				}

			} else {
				// panel editing

				if($this->err($selectedPanelStatus = $this->validateParam("packageStatus", INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED), "No appropriate package status ID given")) break;

				$packageQuery = new MBQuery(
					"GetPackages", "",
					array(
						"packageID" => $packageID,
						"getPackageAttributeData" => 1,

					)
				);
				if(!$this->addErrorsFromResult($packageResult =& $packageQuery->getResult())) break;
				$packageAttributeTable =& $packageResult->getTableFrom(
					array("mbapi", "results", "packages", "package", "packageAttributes", "packageAttribute"),
					"packageAttributeName", "packageAttributeValue"
				);
				$packageAttributes = $packageAttributeTable->getTable();
				$configGroupPackageAttributeTables =& $packageAttributeTable->getAssociatedTablesBy("configGroupID");
				$ourPackageAttributeTable =& $configGroupPackageAttributeTables[$configGroupID];
				$panelAttributes = $ourPackageAttributeTable->getOptions();
				$panelStatus = $panelAttributes["panel_status"];
				$panelStatusID = getAPIStatusID("package", $panelStatus);

				if($panelStatusID != $selectedPanelStatus &&  in_array($this->_subscription->package_type, array(Plan::TYPE_HOSTING, Plan::TYPE_RESELLER))) {
					$subCommand = "";
					switch($selectedPanelStatus) {
						case Subscription::STATUS_ACTIVE:
							$subCommand = "panelUnsuspend";
							break;
						case Subscription::STATUS_SUSPENDED :
							$subCommand = "panelSuspend";
							break;
						case Subscription::STATUS_CANCELED :
							$subCommand = "panelDelete";
							break;
					}

					$panelQuery = new MBQuery(
						"ProcessPanel", $subCommand,
						array(
							'subCommand' => $subCommand,
							'configGroupID' => $configGroupID,
							'packageID' => $packageID,
						)
					);
					$panelXML = $panelQuery->getXMLQuery();

					$panelQuery = new MBQuery(
						"SetSystemQueue", "insert",
						array(
							"systemQueueCommand" => "ProcessPanel",
							"configGroupID" => $configGroupID,
							"packageID" => $packageID,
							"systemQueueCommandInput" => htmlentities($panelXML),
							"systemQueueDateToRun" => mktime(),
							"systemQueueDateFinished" => 0,
						)
					);

					if(!$this->addErrorsFromResult($panelResult =& $panelQuery->getResult())) break;
				}
			}

			if(!$this->goBack()) {
				redirect_url(action_url('packages', 'EditPackage', array("packageID" => $packageID)));
				return $this->hasExecutedSuccessfully();
			}
		} while(false);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _changePrimaryDomain()
    {
        if (is_null($selectedAdditionalOperation = bgpc("packageDomainID_operation"))) {
            return;
        }
        $transaction = new Transaction(DatabaseManager::getConnection());
        if ($selectedPackageDomainId = bgpc("packageDomainID")) {
            $this->_subscription->primary_domain_id = $selectedPackageDomainId;
            if (!in_array($selectedPackageDomainId, $this->_subscription->packageDomainIDs)) {
                $this->_subscription->packageDomainIDs = array_merge($this->_subscription->packageDomainIDs, array($selectedPackageDomainId));
            }
        } else {
        	$this->_subscription->primary_domain_id = null;
        }
		switch($selectedAdditionalOperation) {
			case "add" :
				$newDomainSld = bgpc("packageDomainID_newDomainSLD");
        		$newDomainTld = bgpc("packageDomainID_newDomainTLD");
				if (!strlen($newDomainSld) || !strlen($newDomainTld)) {
					$this->_displayError(blmsg('TRANS_SUBSCRIPTION_ADMIN_EDIT_DOMAIN_ADD_EMPTY'));
					break;
				}
				$domain = $this->_getUserDomain($newDomainSld, $newDomainTld);
				if (in_array($domain->domain_id, $this->_subscription->packageDomainIDs)) {
					$this->_displayError(blmsg('TRANS_SUBSCRIPTION_ADMIN_EDIT_DOMAIN_ADD_EXISTENT', array('DOMAIN' => $domain->name)));
					break;
				}
				if (!in_array($domain->domain_id, $this->_subscription->packageDomainIDs)) {
					$this->_subscription->packageDomainIDs = array_merge($this->_subscription->packageDomainIDs, array($domain->domain_id));
				}
				break;
			case "replace" :
				$replaceDomain = Factory()->PackageDomain(bgpc("packageDomainID_replaceDomainID"));
				if (!in_array($this->_subscription->id, $replaceDomain->packageIDs)) {
					$this->_displayError(blmsg('TRANS_SUBSCRIPTION_ADMIN_EDIT_DOMAIN_REPLACE_FOREIGN', array('DOMAIN' => $replaceDomain->name)));
					break;
				}
				$newDomainSld = bgpc("packageDomainID_replaceDomainSLD");
        		$newDomainTld = bgpc("packageDomainID_replaceDomainTLD");
        		if (!strlen($newDomainSld) || !strlen($newDomainTld)) {
        			$this->_displayError(blmsg('TRANS_SUBSCRIPTION_ADMIN_EDIT_DOMAIN_REPLACE_EMPTY'));
					break;
				}
				$domain = $this->_getUserDomain($newDomainSld, $newDomainTld);
				if (in_array($domain->domain_id, $this->_subscription->packageDomainIDs)) {
					$this->_displayError(blmsg('TRANS_SUBSCRIPTION_ADMIN_EDIT_DOMAIN_REPLACE_EXISTENT', array('DOMAIN1' => $replaceDomain->name, 'DOMAIN2' => $domain->name)));
					break;
				}

				if ($replaceDomain->id == $selectedPackageDomainId) {
					$this->_subscription->primary_domain_id = $domain->domain_id;
				}

				$this->_subscription->packageDomainIDs = array_merge($this->_subscription->packageDomainIDs, array($domain->domain_id));
				$this->_subscription->packageDomainIDs = array_diff($this->_subscription->packageDomainIDs, array($replaceDomain->id));

				// delete domain, if is connected only with this subscription
				if(1 == count($replaceDomain->packageIDs)) {
					$replaceDomain->delete();
				}

				break;
			case "remove" :
				$removeDomain = Factory()->PackageDomain(bgpc("packageDomainID_removeDomainID"));
				if (!in_array($this->_subscription->id, $removeDomain->packageIDs)) {
					$this->_displayError(blmsg('TRANS_SUBSCRIPTION_ADMIN_EDIT_DOMAIN_REMOVE_FOREIGN', array('DOMAIN' => $removeDomain->name)));
					break;
				}
				if ($selectedPackageDomainId == $removeDomain->id) {
					$this->_displayError(blmsg('TRANS_SUBSCRIPTION_DOMAIN_REMOVAL_FORBIDDEN_MSG'));
					break;
				}

				$this->_subscription->packageDomainIDs = array_diff($this->_subscription->packageDomainIDs, array($removeDomain->id));

				// delete domain, if is connected only with this subscription
				if (1 == count($removeDomain->packageIDs)) {
					$removeDomain->delete();
				}
				break;
		}

		$transaction->commit();
    }

    private function _getUserDomain($domainSld, $domainTld)
    {
        $subscriptions = Factory()->SubscriptionList(array('client_id' => $this->_subscription->client_id));
        foreach ($subscriptions as $subscription) {
            foreach ($subscription->PackageDomainList() as $domain) {
                if (($domainSld == $domain->domain_sld) && ($domain->domain_tld == $domainTld)) {
                    return $domain;
                }
            }
        }

        $domain = Factory()->PackageDomain();
		$domain->domain_sld = $domainSld;
		$domain->domain_tld = $domainTld;
		$domain->domain_idn_language = "en";
		$domain->update();
		return $domain;
    }

    private function _displayError($message)
    {
    	billing_put_error($message);
		if ('serverassociate' == bgpc('task') || 'panelAction' == bgpc('task') || '' != bgpc('subscriptionAction')) {
			redirect("packages.php", "ShowPackageDetails", array("packageID" => $this->_subscription->id));
		} else {
			redirect("packages.php", "EditPackage", array("packageID" => $this->_subscription->id));
		}
    }

    private $_subscription;
}