<?php
require_once ("lib-action/action.php");

use Billing\Manager\CryptManager;

class RunPaymentQueueResponse extends MB_Action
{
	public function execute()
    {
		parent::execute();

		$this->smartyVars["hideEventStatusBar"] = 1;
		$this->smartyVars["queueRunTime"] = time();

		if (checkdate($_REQUEST["laterDateMonth"],$_REQUEST["laterDateDay"],$_REQUEST["laterDateYear"]) && (int) $_REQUEST["runWhen"]==2) {
			if (intval($_REQUEST["laterDateHour"]) <= 23) {
				$laterDateHour = (int) $_REQUEST["laterDateHour"];
			} else {
				$laterDateHour = date("h");
			}
			if (intval($_REQUEST["laterDateMinute"]) <= 59) {
				$laterDateMinute = (int) $_REQUEST["laterDateMinute"];
			} else {
				$laterDateMinute = date("i");
			}
			$laterDate = mktime($laterDateHour,$laterDateMinute,0,$_REQUEST["laterDateMonth"],$_REQUEST["laterDateDay"],$_REQUEST["laterDateYear"]);
		} else {
			$laterDate = mktime()-300;
		}

		$timeData = explode(";", date("i;G;j;n;Y", $laterDate));

		if ( (int) $_REQUEST["runWhen"] ){
			if("" == bgpc("encryptionKey")) {
				billing_put_error(blmsg("TRANS_ENCRYPTION_NO_LEK_PIN_GIVEN_ERROR_MSG"));
				redirect("paymentqueue.php", "RunPaymentQueue");
			} else {
				if (CryptManager::isEncryptionEnabled()) {
					try {
						CryptManager::checkEncryptionKey(bgpc("encryptionKey"), getCurrentSession()->user_id);
					} catch(ProductException $e) {
						billing_put_error($e->getMessage());
						redirect("paymentqueue.php", getActionID("RunPaymentQueue"));
					}
				}
			}

			// its not running our process, so lets set the queue command and start it from the tpl or just say we set it
			$vendorID = getCurrentSession()->getVendorId();
			$vendorIDParam = !is_null($vendorID) ? $vendorID : "NULL";
			$key = (CryptManager::MODE_LEK == CryptManager::getEncryptionMode()) ?
				"" :
				"<encryptedKey>" . utf8_xml_entity_encode(CryptManager::encrypt(bgpc("encryptionKey"))) . "</encryptedKey>"
			;
			$processQueueXML = "
				<mbapi>
					<command>ProcessPaymentQueue</command>
					<params>
						<manualRun>1</manualRun>
						<runPaymentQueue>1</runPaymentQueue>
						<numInvoicesToCharge>3</numInvoicesToCharge>
						<transactionAction>".secureInput($_REQUEST["transactionAction"])."</transactionAction>
						{$key}
						<chargeLesserAmounts>".(((int)$_REQUEST["chargeLesserAmounts"]>0)?1:0)."</chargeLesserAmounts>
						<vendorID>{$vendorIDParam}</vendorID>
					</params>
				</mbapi>
				"; // run everything in the queue with a date charged of 0

			$dateToRun = mktime($timeData[1], $timeData[0], 0, $timeData[3], $timeData[2], $timeData[4]);
			$cmdArgs = array(
				"systemQueueCommand" => "ProcessPaymentQueue",
				"systemQueueCommandInput" => $processQueueXML,
				"systemQueueMinute" => $timeData[0],
				"systemQueueHour" => $timeData[1],
				"systemQueueDay" => $timeData[2],
				"systemQueueMonth" => $timeData[3],
				"systemQueueYear" => $timeData[4],
				"systemQueueIsRecurring" => 0,
				"systemQueueDateCreated" => time(),
				"systemQueueDateToRun" => $dateToRun,
				"systemQueueDateStarted" => 0,
				"systemQueueDateFinished" => 0
			);
			if (!is_null($vendorID)) {
				$cmdArgs["vendorID"] = $vendorID;
			}
			$query = $this->getMBAPIQuery("SetSystemQueue", "insert", $cmdArgs);
			$data = dispatchMBAPI($query, 1);
			$this->smartyVars["systemQueueID"] = $data["mbapi"][0]["header"][0]["lastInsertID"][0];

			$this->smartyVars["runWhen"] = (int)$_REQUEST["runWhen"];
			$this->smartyVars["runDate"] = $laterDate;
			$this->smartyVars["showStatus"] = "yes";

		} else { // nope we didnt
			$this->smartyVars["showStatus"] = "yes";
			$this->smartyVars["queueRunTime"] = mktime(); //$queueItem["mbapi"][0]["results"][0]["systemQueue"][0]["systemQueueEvent"][0]["systemQueueDateToRun"][0];
		}

		$this->addTemplate("paymentqueue/RunPaymentQueueResponse", $this->smartyVars);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}