<?php

class Products_PlanOverviewController extends Billing_Zend_Controller_Action_AdminController
{
	public function init()
	{
		parent::init();
	}

	public function indexAction()
	{
        try {
            if (!is_null($id = $this->getRequest()->getParam('id'))) {
                $plan = Factory()->Plan($id);
            } elseif(!is_null($guid = $this->getRequest()->getParam('productGUID'))) {
                $plan = Factory()->Plan($guid, 'guid');
            } elseif(!is_null($externalId = $this->getRequest()->getParam('planExternalID'))) {
                $plan = Factory()->Plan($externalId, 'external_id');
            } else {
                throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, $this->_translator->translate("TRANS_NO_PLAN_ID_OR_GUID"));
            }
        } catch (ProductException $e) {
            $this->_redirectWithMessage($this->view->ActionUrl('products', 'ShowProducts'), $e->getMessage(), "error");
        }

		$this->_plan = $plan;
        $this->_problems['messages'][$this->_getProblemTitle()] = array();
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => billing_safetyhtml($plan->product_name))));

		$problemMessage = array();
        require_once 'object_managers/ProductManager.php';
		ProductManager::isPlanValidForStores($plan, $plan->orderFormIDs, $this->_problems['messages'][$this->_getProblemTitle()]);
        
		$activeProductVariants = array();
		foreach ($plan->ProductVariantList() as $productVariant) {
			if ($productVariant->product_variant_active) {
				$activeProductVariants[$productVariant->id] = $productVariant;
			}
		}

		$activeCycles = array();
		foreach ($plan->cycleIDs as $cycleId) {
			$activeCycles[$cycleId] = true;
		}

		$this->view->type = 'TRANS_PRODUCT_TYPE_NAME_' . $plan->product_type;
		$this->view->subscribers = $plan->getPackagesCount();

		require_once 'objects/ProductVariantPrice.php';
		$activeCurrencies = array();
		foreach (ProductManager::getPricesData($plan->id) as $cycleId => $variants) {
			if (!isset($activeCycles[$cycleId])) {
				continue;
			}
			foreach ($variants as $variantId => $currencies) {
				if (!isset($activeProductVariants[$variantId])) {
					continue;
				}
				foreach ($currencies as $code => $prices) {
					if (isset($prices[ProductVariantPrice::TYPE_GENERAL])) {
						$activeCurrencies[$code] = true;
					}
				}
			}
		}
		$stores = $plan->OrderFormList();
		$actualCurrencies = array();
		$missedCurrencies = array();
		foreach ($stores as $store) {
			$actualCurrencies[$store->currency_id] = true;
			if (!isset($activeCurrencies[$store->currency_id]) && !isset($missedCurrencies[$store->currency_id])) {
				$missedCurrencies[$store->currency_id] = true;
			}
		}
		$this->view->stores = $stores;

		require_once 'objects/Product.php';
		if (in_array($plan->product_type, array(Product::TYPE_HOSTING, Product::TYPE_RESELLER))) {
			$serverGroupIds = $plan->serverGroupIDs;

            if (sizeof($serverGroupIds) == 1) {
				$serverGroupId = $serverGroupIds[0];
				$attributes = array();
				$planAttributesList = $plan->ProductAttributesList(array('server_group_id' => $serverGroupId));
				if ($planAttributes = array_shift($planAttributesList)) {
					$displayableAttributes = array(
						'domainDiskSpace',
						'domainMaxTraffic',
						'domainMaxSites',
						'domainMaxMailbox',
						'domainMaxMaillists',
						'domainMaxMYSQLDB',
                        'domainMaxMSSQLDB',
                        'limit-pleskweb.domains_number',
                        'limit-disc_space',
                        'limit-pleskweb.traffic',
                        'limit-pleskweb.mysql.db_number'
					);
                    if (Product::TYPE_RESELLER == $plan->product_type) {
                        array_unshift($displayableAttributes, "max_cl");
                    }
					foreach ($displayableAttributes as $name) {
						if (isset($planAttributes[$name]) && $planAttributes[$name]->product_attribute_value) {
							$item = array('value' => $planAttributes[$name]->product_attribute_value);
                            $pleskUnit = $name . 'UNIT';
                            $poaUnit = str_replace(array('limit-pleskweb.', 'limit-'), 'unit-', $name).'.unit';
							if (isset($planAttributes[$pleskUnit])) {
								$item['unit'] = $planAttributes[$pleskUnit]->product_attribute_value;
							} else if (isset($planAttributes[$poaUnit])) {
								$item['unit'] = $planAttributes[$poaUnit]->product_attribute_value;
							}
							$attributes[$name] = $item;
						}
					}
				}
				$this->view->attributes = $attributes;
				$this->view->attributesCount = sizeof($planAttributes);
				$this->view->displayAttributes = sizeof($attributes) ? true : false;
			} else {
				$this->view->displayAttributes = false;
			}

		} else {
			$this->view->displayAttributes = false;
		}

		if ($plan->isAddon()) {
			$this->view->parents = $plan->ParentList();
		} else {
			// Addons
			$addons = $plan->ChildrenList();
			usort($addons, array($this, '_sortAddons'));
			$this->view->addons = $addons;

			// Additional services
			if ($this->view->allowedAdditionalServices = in_array($plan->product_type, array(Product::TYPE_HOSTING, Product::TYPE_RESELLER))) {
				$this->view->additionalServices = $plan->AdditionalServiceList();
			}
		}

		$prices = array();
		require_once 'objects/ProductVariantPrice.php';
		$productVariantList = $plan->ProductVariantList();
		if ($variant = array_shift($productVariantList)) {
			foreach ($plan->CycleList() as $cycle) {
				$tmp = $variant->ProductVariantPriceList(
					array(
						'cycle_id' => $cycle->id,
						'product_variant_price_type' => ProductVariantPrice::TYPE_GENERAL
					)
				);
				$variantPrices = array();
				foreach ($tmp as $price) {
					if (isset($actualCurrencies[$price->currency_id])) {
						$variantPrices[] = $price;
					}
				}
				if ($variantPrices) {
					$prices[$cycle->cycle_display_name] = $variantPrices;
				}
			}
		}
		$this->view->prices = $prices;
		$this->view->displayPrices = $variant && !sizeof($productVariantList) ? true : false;

		if ($variant && $discount = $variant->CycleDiscountGroup) {
			$this->view->discount = $discount;
		}

		$this->view->coupons = $plan->CouponList();
		$this->view->preappliedCouponIDs = array();
		foreach($plan->preappliedCouponList() as $coupon) {
			$this->view->preappliedCouponIDs[] = $coupon->id;
		}

		require_once 'UI/Page/UISelectProductTypePage.php';
		$toolButtons = array(
			'settings' => array(
				'label' => $plan->isAddon() ? 'TRANS_ADDON_OVERVIEW_TOOLBAR_SETTINGS' : 'TRANS_PLAN_OVERVIEW_TOOLBAR_SETTINGS',
				'class' => 'sb-plan-settings',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'edit', array('id' => $plan->id))
			),
			'ledger' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_LEDGER',
				'class' => 'sb-plan-ledger',
				'href' => $this->view->actionUrl('products', 'EditLedgerGroups', array('productID' => $plan->id)),
				'separate' => true
			),
			'sync' => array(
				'label' => 'TRANS_SYNC_CMD',
				'class' => 'sb-sync',
				'href' => $this->view->actionUrl('products', 'ShowProducts', array("sync" => 1, 'id[]' => $plan->id, "redirectAction" => "ShowProductDetails")),
				'separate' => true
			),
			'clone' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_CLONE',
				'class' => 'sb-plan-clone',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'clone', array('id' => $plan->id)),
				'separate' => false
			),
		);

		if(0 == $plan->getPackagesCount()) {
			$toolButtons['remove'] = array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_REMOVE',
				'class' => 'sb-plan-remove',
				'href' => $this->view->actionUrl('products', 'RemoveProduct', array('productID' => $plan->id)),
				'separate' => true
			);
		}

		if (($reseller = getCurrentSession()->getReseller()) && !$reseller->hasPermission(Reseller::PERMISSION_ACCOUNTING)) {
			unset($toolButtons['ledger']);
		}
        $this->_compareCycles();

		$this->view->problems = $this->_problems;

		$this->view->toolButtons = $toolButtons;

		$this->view->isAddon = $plan->isAddon();

        $addons = $plan->addonIDs;
        if (!$plan->isAddon() && (\Billing\Object\Plan::ADDON_NOT_NEEDED == $plan->product_addon_required) && !empty($addons)) {

            $this->_status->addWarning($this->_translator->translate('TRANS_PLAN_OVERVIEW_PROBLEM_NO_OFFER_ADDONS'));
        }

		if ($plan->isSyncable() && !is_null($plan->sync_error)) {
			$transKey = $plan->isAddon() ? "TRANS_SYNC_THIS_PLAN_ADDON_ERROR_MSG" : "TRANS_SYNC_THIS_PLAN_ERROR_MSG";
			$this->_status->addError(blmsg($transKey, array("ERROR" => nl2br(billing_safetyhtml($plan->sync_error)))));
		}
    }

    private function _compareCycles()
    {
        if (!$this->_plan->isAddon()) {
            foreach ($this->_plan->addonIDs as $addonId) {
            	
                $addon = Factory()->Plan($addonId);
                
                $addonCycles = array();
                foreach ($addon->CycleList() as $addonCycle) {
                    $addonCycles[$addonCycle->cycle_id] = $addonCycle->cycle_display_name;
                }

                foreach($this->_plan->CycleList() as $planCycle) {
                    if (!isset($addonCycles[$planCycle->cycle_id])) {
                        $this->_problems['messages']["TRANS_PLAN_ADDONS_SOME_CYCLES_NOT_COMPARE_PLAN"][] = $this->_translator->translate("TRANS_PLAN_ADDON_CYCLE_NOT_DEFINE_ADDON_MSG",
                            array(
                                "CYCLE" => $this->_translator->translate($planCycle->cycle_display_name),
                                "ADDON" => $addon->product_name
                            )
                        );
                    }
                }
            }
        } else {
            $addonCycles = array();
            foreach ($this->_plan->CycleList() as $addonCycle) {
                $addonCycles[$addonCycle->cycle_id] = $addonCycle->cycle_display_name;
            }

            foreach($this->_plan->ParentList() as $plan) {
                foreach ($plan->CycleList() as $planCycle) {
                    if (!isset($addonCycles[$planCycle->cycle_id])) {
                        $this->_problems['messages']["TRANS_PLAN_ADDONS_SOME_CYCLES_NOT_COMPARE_ADDON"][] = $this->_translator->translate("TRANS_PLAN_ADDON_CYCLE_NOT_DEFINE_PLAN_MSG",
                            array(
                                "CYCLE" => $this->_translator->translate($planCycle->cycle_display_name),
                                "PLAN" => $plan->product_name
                            )
                        );
                    }
                }
            }
        }
    }

    private function _getProblemTitle()
    {
        return ($this->_plan->isAddon()) ? 'TRANS_ADDON_OVERVIEW_OVERVIEW_PROBLEM' : 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM';
    }

    private function _sortAddons($a, $b)
    {
    	if ($a->product_addon_group_id > $b->product_addon_group_id) {
    		return 1;
    	} else if ($a->product_addon_group_id < $b->product_addon_group_id) {
    		return -1;
    	} else {
    		return 0;
    	}
    }
    
    private $_plan = null;
    private $_problems = null;
}