<?php

require_once("lib-action/action.php");

class ShowDashboard extends MB_Action
{
	public function execute()
	{
        parent::execute();
		$contact = Factory()->Contact(getCurrentSession()->user_id);
		$clientID = $contact->Customer->id;
		$this->mergeClientData();

		// ---------------------------------------------------
		// Get DUE invoices ...
		// ---------------------------------------------------
		$this->smartyVars["invoices"] = array(); // init
		$params = array(
			"clientID" => $clientID,
			"getDueInvoicesOnly" => 1,
			"getInvoiceLineItemsData" => 0
		);
		require_once("lib-mbapi/mbapi.php");
		$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0, 5);
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {
			// set master invoices data array
			$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));
			// build invoice line item data
			if($this->smartyVars["invoices"]) {
				for ($i = 0; $i < count($this->smartyVars["invoices"]); $i++){
					if (!$this->smartyVars["invoices"][$i]["lineItems"]["lineItem"][0]["clientID"]){
						$this->smartyVars["invoices"][$i]["lineItems"]["lineItem"] = array($this->smartyVars["invoices"][$i]["lineItems"]["lineItem"]);
					}
				}
			}

			// set master invoices line item data array
			$lineItems = array();
			$c = count($results["invoices"][0]["invoice"]);
			for($i = 0; $i < $c; $i++) {
				// leave this uncollapsed so that the template code can work with both singular and plural results;
				// see the comment in the template for details
				$lineItemsData =& $results["invoices"][0]["invoice"][$i]["lineItems"][0]["lineItem"];
				$lineItems[$results["invoices"][0]["invoice"][$i]["invoiceID"][0]] = collapseArrayElements(protectArray($lineItemsData));
			}
			$this->smartyVars["lineItems"] = $lineItems;
		}

		// ---------------------------------------------------
		// Get ACTIVE packages ...
		// ---------------------------------------------------
		$suppressFeePackages = ((int)$_REQUEST["showFees"])?0:1;

		$this->smartyVars["packages"] = correctArray(grabResultsFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetPackages",
				"params" => array(
					"clientID" => $clientID,
					"getBillablePackagesOnly" => 1,
					"getDomainData" => 1,
					"suppressFeePackages" => $suppressFeePackages,
				),
			),
		)));

		$this->smartyVars["packageStatuses"] = getStatuses("package");
		require_once("lib-billing/include/config/internal/cycles.php");
		$this->smartyVars["cycles"] = getCyclesArray();
        $this->smartyVars["overuses"] = \Billing\Manager\CustomerManager::getTotalOveruse(Factory()->Customer($clientID));

		$uir_packages = Factory()->SubscriptionList(array(
			"client_id" => $clientID,
			"package_status" => getAPIStatusID("package", "user_input_required"),
		));
		if (0 < count($uir_packages))
		{
			$messages = array(blmsg("TRANS_UIR_YOU_HAVE_PACKAGES_THAT_REQUIRE_ADDITIONAL_DATA"));
			foreach ($uir_packages as $package)
			{
				$domain = is_null($package->PrimaryDomain)
					? ""
					: " ({$package->PrimaryDomain->domain_sld}.{$package->PrimaryDomain->domain_tld})";

				$message = blmsg("TRANS_UIR_PACKAGE_NAME", array(
					"packageID" => $package->package_id,
					"packageName" => $package->package_name . $domain,
				));
				$message = "<a href=\"".action_url("packages", "ShowClientServiceInputForm", array("packageID" => $package->package_id))."\">{$message}</a>";
				$messages[] = $message;
			}
			$this->smartyVars["messages"] = UIFactory()->WarningMessage(join("<BR/>", $messages))->getHTML();
		}

		if ( 0 == $contact->contact_primary && $contact->contact_username != $contact->contact_email ) {
			$this->smartyVars["warningMessages"][] = blmsg(
				"TRANS_CLIENT_CONTACT_EMAIL_USERNAME_DIFFERENT_CONTACT_WELCOME_MSG",
				array("URL" => action_url("profile", "EditClientContact"))
			);
		}

		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("dashboard/ShowDashboard", $this->smartyVars));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
