<?php
use \Billing\Manager\CryptManager;
use \Billing\Manager\PaymentGatewayManager;
use \Billing\Manager\PaymentManager;
use \Billing\Object\BillingAccount;
use \Billing\Object\Contact;
use \Billing\Object\PaymentGateway;

require_once("lib-action/action.php");

class PayInvoices extends MB_Action
{
	public function execute()
    {
		parent::execute();
        $this->smartyVars["encryption"] = CryptManager::isEncryptionEnabled();
		$this->smartyVars["nolek"] = (CryptManager::isLekEnabled() ? 0 : 1);

        $this->_contact = Factory()->Contact(getCurrentSession()->user_id);
        $this->_setBillingAccounts();

		for ($i=date("Y"); $i<date("Y")+21; $i++){
			$this->smartyVars["cardYears"][$i] = $i;
		}
		$this->smartyVars["cardTypes"] = PaymentGateway::card_types();
		$this->smartyVars["payTotal"] = 0;

		// ---------------------------------------------------
		// Get DUE invoices ...
		// ---------------------------------------------------
		$this->smartyVars["invoices"] = array(); // init
		$this->smartyVars["invoiceGroups"] = array();
		$params = array(
			"clientID" => $this->_contact->client_id,
			"getDueInvoicesOnly" => 1,
		);
		if ((int)$_REQUEST["invoiceID"]){
			$params["invoiceID"] = $this->smartyVars["invoiceID"] = (int)$_REQUEST["invoiceID"];
		}
		$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0);
		$data = dispatchMBAPI($query, 1);
		if ( 0 !== $this->addMBAPIErrors($data) ) {
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		// set master invoices data array
		$results = $data["mbapi"][0]["results"][0];
		$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));

		if (empty($this->smartyVars["invoices"])) {
			if (isset($params["invoiceID"])) {
				throw new ProductException(
					ProductException::E_FAIL_OPERATION,
					blmsg("TRANS_PAYMENT_INVOICE_ALREADY_PAYED_ERROR_MSG", array("INVOICE_ID" => $params["invoiceID"]))
				);
			} else {
				throw new ProductException(ProductException::E_FAIL_OPERATION, blmsg("TRANS_PAYMENT_NO_INVOICES_TO_PAY_ERROR_MSG"));
			}
		}

		// build invoice line item data
		if($this->smartyVars["invoices"]) {
			for ($i = 0; $i < count($this->smartyVars["invoices"]); $i++){
				$this->smartyVars["invoices"][$i]["invoiceAmountDue"] = $this->smartyVars["invoices"][$i]["invoiceAmount"]-$this->smartyVars["invoices"][$i]["invoiceAmountPaid"];
				$this->smartyVars["invoiceGroups"][$this->smartyVars["invoices"][$i]["orderFormID"]]["invoices"][] =& $this->smartyVars["invoices"][$i];
				$thisdue = $this->smartyVars["invoices"][$i]["invoiceAmount"]-$this->smartyVars["invoices"][$i]["invoiceAmountPaid"];
				if ($thisdue<0)$thisdue = 0;
				$this->smartyVars["invoiceGroups"][$this->smartyVars["invoices"][$i]["orderFormID"]]["payTotal"] += $thisdue;
			}
		}

		$order_form_ids = array_keys($this->smartyVars["invoiceGroups"]);
		$order_forms = Factory()->OrderFormList(array("order_form_id" => $order_form_ids));
		foreach ( $order_forms as $order_form ) {
			if ( !is_null($order_form->PrimaryGateway) ) {
				$this->smartyVars["invoiceGroups"][$order_form->id]["hasPrimary"] = 1;
				$this->smartyVars["invoiceGroups"][$order_form->id]["cardTypes"] = $order_form->PrimaryGateway->acceptedCardTypes;
			}
			if (!is_null($order_form->HSPCBankTransferGateway) && CryptManager::MODE_NONE != CryptManager::getEncryptionMode()) {
				$this->smartyVars["invoiceGroups"][$order_form->id]["hasHSPCBankTransfer"] = 1;
                $plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $order_form->HSPCBankTransferGateway->payment_gateway_id));
                $billing_account = Factory()->BillingAccount();
                $billing_account->payment_gateway_id = $order_form->HSPCBankTransferGateway->payment_gateway_id;
                $adapter_input_fields = $plugin->getPaymentMethodAdapter($billing_account)->getInputFields(true);
                foreach ($adapter_input_fields as $fieldName => $fieldOpts) {
                    $field = array(
                        'name' => $fieldName,
                        'label' => blmsg($fieldOpts[0]),
                        'value' => blmsg($fieldOpts[1]),
                        'required' => true,
                        'description' => blmsg($fieldOpts[2]),
                    );
                    $this->smartyVars["invoiceGroups"][$order_form->id]["bankTransferFields"][] = $field;
                }
			}
			$this->smartyVars["invoiceGroups"][$order_form->id]["secondary"] = $order_form->ThirdPartyGatewaysList();
		}
		$this->smartyVars["secondaryGatewayNames"] = array();
		foreach ((array)$this->smartyVars["invoices"] as $k=>$invoice){
			if ( !isset($this->smartyVars["invoiceGroups"][$invoice["orderFormID"]]["secondary"]) ) {
				continue;
			}
			$secondaries = $this->smartyVars["invoiceGroups"][$invoice["orderFormID"]]["secondary"];
			foreach ( $secondaries as $secondary_gateway ) {
				try {
					if (!PaymentGatewayManager::isOperationSupported($secondary_gateway, PaymentGatewayManager::OPERATION_CHARGE)) {
						continue;
					}
					$billingAccount = reset(Factory()->BillingAccountList(array(
						"billing_account_type" => BillingAccount::TYPE_THIRD_PARTY_GATEWAY,
						"client_contact_id" => getCurrentSession()->user_id,
						"payment_gateway_id" => $secondary_gateway->id,
					)));
					if (false === $billingAccount) {
						$billingAccount = Factory()->BillingAccount();
						$billingAccount->billing_account_type = BillingAccount::TYPE_THIRD_PARTY_GATEWAY;
						$billingAccount->client_contact_id = getCurrentSession()->user_id;
						$billingAccount->payment_gateway_id = $secondary_gateway->id;
						$billingAccount->update();
					}
					list($chargeRequest) = PaymentManager::getChargeRequests(Factory()->Invoice($invoice["invoiceID"]), $billingAccount, true);
					$this->smartyVars["invoices"][$k]["secondary"][$secondary_gateway->moduleName][] = $chargeRequest;
				} catch (ProductException $e) {
					dbg($e->getMessage());
					// skip gateway
				}
			}
			if ($this->smartyVars["invoices"] && $this->smartyVars["invoices"][$k] && is_array($this->smartyVars["invoices"][$k])){
				$this->smartyVars["secondaryGatewayNames"][$invoice["orderFormID"]] = array_keys((array)$this->smartyVars["invoices"][$k]["secondary"]);
			}
		}

		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("invoices/PayInvoices", $this->smartyVars));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _setBillingAccounts()
    {
        $this->smartyVars["billingAccounts"] = $this->smartyVars["billingAccountTypes"] = array();
        foreach ($this->_contact->BillingAccountList(array('billing_account_active' => 1)) as $account) {
            switch ($account->billing_account_type) {
			case BillingAccount::TYPE_CREDIT_CARD:
			case BillingAccount::TYPE_ECHECK:
			case BillingAccount::TYPE_HSPC_BANK_TRANSFER:
				if (BillingAccount::TYPE_HSPC_BANK_TRANSFER == $account->billing_account_type && CryptManager::MODE_NONE == CryptManager::getEncryptionMode()) {
					break;
				}
				$this->smartyVars["billingAccounts"][$account->billing_account_id] = $account->getAccountName();
				$this->smartyVars["billingAccountTypes"][$account->billing_account_id] = $account->billing_account_type;
				break;
            }
        }
        $this->smartyVars["billingAccountCC"] = BillingAccount::TYPE_CREDIT_CARD;
        $this->smartyVars["billingAccountEcheck"] = BillingAccount::TYPE_ECHECK;
    }

    private $_contact = null;
}
