<?php
require_once("lib-action/action.php");
use Billing\Manager\BillingAccountManager;
use Billing\Manager\CryptManager;
use Billing\Manager\InvoiceManager;
use Billing\Manager\PaymentManager;
use Billing\Manager\PaymentTransactionManager;
use Billing\ModuleTool;
use Billing\Object\BillingAccount;
use Billing\Object\Invoice;
use Billing\Object\PaymentGateway;
use Billing\Object\PaymentTransaction;

class PayInvoicesResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$this->_contact = Factory()->Contact(getCurrentSession()->user_id);

		$invoiceID = (int)bgpc("invoiceID");
		$invoiceIDs = bgpc("invoiceIDs");
		$billingAccountID = bgpc("billingAccountID");
		$paymentAmount = doubleval(bgpc("payTotal"));

		$handle3dSecureResponse = false;
		if (!is_null($paymentTransactionId = bgpc("paymentTransactionId"))) {
			$transaction = Factory()->PaymentTransaction($paymentTransactionId);
			if ($transaction->BillingAccount->client_contact_id == $this->_contact->id &&
				PaymentTransaction::STATUS_PROCESSING == $transaction->auth_return
			) {
				// handle 3D secure response
				$invoiceIDs = $transaction->invoiceIDs;
				$invoiceID = (1 == count($invoiceIDs)) ? $invoiceIDs[0] : 0;
				$billingAccountID = $transaction->billing_account_id;
				$paymentAmount = doubleval($transaction->transaction_amount);

				$processingParams = $transaction->processingParams;
				$processingParams->backRedirectParams = http_build_query(bgpc_array($_POST));
				$handle3dSecureResponse = true;
			}
		}

		$oid = $invoiceID ? "<invoiceID>".$invoiceID."</invoiceID>" : "";
		$invoiceXML = "
			<mbapi>
				<command>GetInvoices</command>
				<params>
					<clientID>".$this->_contact->client_id."</clientID>
					<getDueInovicesOnly>1</getDueInovicesOnly>
					<getInvoiceLineItemsData>1</getInvoiceLineItemsData>
					".$oid."
					<orderFormID>".((int)$_REQUEST["orderFormID"])."</orderFormID>
				</params>
			</mbapi>
		";
		require_once("lib-mbapi/mbapi.php");
		$invoices = dispatchMBAPI($invoiceXML, 1);
		$invoices = $invoices["mbapi"][0]["results"][0];
		if (isset($invoices["invoices"][0]["invoice"])){
			$invoices = correctArray($invoices["invoices"][0]["invoice"]);
		}
		$validInvoiceIDs = array();
		foreach ($invoices as $invoice){
			$validInvoiceIDs[] = $invoice["invoiceID"];
		}
		$newInvoiceIDs = array();
		if (isset($invoiceIDs)){
			foreach ($invoiceIDs as $v){
				if (in_array($v, $validInvoiceIDs)){
					$newInvoiceIDs[] = $v;
				}
			}
			$invoiceIDs = $newInvoiceIDs;
		} else {
			dbg("redirected prematurely");
			redirect("invoices.php?action=".getActionID("PayInvoices"));
			return 1;
		}

		$newInvoices = array();
		foreach ($invoices as $invoice){
			if (@in_array($invoice["invoiceID"], $invoiceIDs)){
				$newInvoices[] = $invoice;
			}
		}
		$invoices = $newInvoices;
		if ( 0 == count($invoices) ) {
			return $this->paymentError(blmsg("TRANS_INVOICES_ALREADY_PAID_FOR"));
		}

		if ( !isset($_REQUEST["orderFormID"]) ) {
			return $this->paymentError(blmsg("TRANS_PAYMENT_ERROR_MESSAGE", array("MESSAGE" => "Missing parameter 'orderFormID'")));
		}
		$this->_store = Factory()->Store(bgpc("orderFormID"));
		if ( is_null($this->_store->PrimaryGateway) && (is_null($this->_store->HSPCBankTransferGateway))) {
			return $this->paymentError(blmsg("TRANS_PAYMENT_ERROR_MESSAGE",
				array("MESSAGE" => "Order form #{$this->_store->id} '{$this->_store->order_form_name} has not primary and Bank Transfer payment gateway assigned'"))
			);
		}
        switch($billingAccountID) {
            case 'bt':
                $billingAccount = Factory()->BillingAccount();
                $billingAccount->client_contact_id = $this->_contact->id;
                $billingAccount->billing_account_type = BillingAccount::TYPE_HSPC_BANK_TRANSFER;
                $client = Factory()->Customer($this->_contact->client_id);
                $paymentGatewayId = $this->_store->HSPCBankTransferGateway->payment_gateway_id;
                if (!$client->hasSameVendor(Factory()->PaymentGateway($this->_store->HSPCBankTransferGateway->payment_gateway_id))) {
                    throw new ForbiddenOperationException();
                }
                CryptManager::setDefaultKey();
                $billingAccount->payment_gateway_id = $paymentGatewayId;
                $plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $paymentGatewayId));
                $adapter = $plugin->getPaymentMethodAdapter($billingAccount);
                $adapter->collectInputFields();
                if (HSPC_PM_Adapter::VALIDATION_OK != ($adapter_validation = $adapter->validate())) {
                    throw new ProductException(ProductException::E_OBJECT_HAS_INVALID_ATTRS, $adapter_validation);
                }
                $adapter->save();
                break;
            case 'cc':
                $billingAccount = $this->_createCreditCardBillingAccount();
                break;
            default:
                $billingAccount = $this->_getBillingAccountById($billingAccountID);
                break;
        }

		if (!$handle3dSecureResponse) {
			// create transaction
			$transaction = Factory()->PaymentTransaction();
			$transaction->auth_return = PaymentTransaction::STATUS_PROCESSING;
			$transaction->transaction_amount = $paymentAmount;
			$transaction->currency_id = $this->_store->currency_id;
			$transaction->billing_account_id = $billingAccount->id;
			$transaction->invoiceIDs = $invoiceIDs;
			switch ($billingAccount->billing_account_type) {
			case BillingAccount::TYPE_CREDIT_CARD:
				$transaction->transaction_payment_method = PaymentTransaction::PAYMENT_CCSINGLE;
				$transaction->payment_gateway_id = $this->_store->PrimaryGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_PAYMENT_TRANSACTION_PROCESSING_NOTE");
				break;
			case BillingAccount::TYPE_ECHECK:
				$transaction->transaction_payment_method = PaymentTransaction::PAYMENT_ECHECK;
				$transaction->payment_gateway_id = $this->_store->PrimaryGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_PAYMENT_TRANSACTION_PROCESSING_NOTE");
				break;
			case BillingAccount::TYPE_HSPC_BANK_TRANSFER:
				$transaction->transaction_payment_method = PaymentTransaction::PAYMENT_HSPCBANKTRANSFER;
				$transaction->payment_gateway_id = $billingAccount->PaymentGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_HSPC_BATCH_FILE_TRANSACTION_INPROGRESS");
				break;
			default:
				throw new Exception("Unsuitable billing account type: {$billingAccount->billing_account_type}");
			}
			PaymentTransactionManager::update($transaction);
		}

		$processingParams = $transaction->processingParams;
		$processingParams->shopperIpAddress = $_SERVER["REMOTE_ADDR"];
		$processingParams->shopperSessionId = getCurrentSession()->id;
		$processingParams->shopperAcceptHeader = $_SERVER["HTTP_ACCEPT"];
		$processingParams->shopperUserAgentHeader = $_SERVER["HTTP_USER_AGENT"];
		$processingParams->backRedirectUrl = \Billing\currentUrl(array('paymentTransactionId' => $transaction->id));

		// process transaction
		switch ($transaction->PaymentGateway->payment_gateway_type) {
		case PaymentGateway::TYPE_PRIMARY:
		case PaymentGateway::TYPE_HSPC_CREDIT_CARD:
			if ($handle3dSecureResponse) {
				PaymentManager::process($transaction);
			} else if (CryptManager::isEncryptionEnabled() && "auth" == Settings(getCurrentSession()->getVendorId())->paymentQueue_transactionAction) {
				PaymentManager::auth($transaction);
			} else {
				PaymentManager::charge($transaction);
			}
			break;
		case PaymentGateway::TYPE_HSPC_BANK_TRANSFER:
			require_once("object_managers/BatchFileManager.php");
			BatchFileManager::createTransaction($transaction->id);
			break;
		default:
			throw new Exception("Unsuitable payment gateway type: {$transaction->PaymentGateway->payment_gateway_type}");
		}

		switch ($transaction->auth_return) {
		case PaymentTransaction::STATUS_AUTHORIZED_CHARGE:
		case PaymentTransaction::STATUS_SUCCESSFUL_CHARGE:
			PaymentManager::takePayment($transaction);
			$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_ACCEPTED");
			break;
		case PaymentTransaction::STATUS_PROCESSING:
			if (!$handle3dSecureResponse && isset($processingParams->redirectUrl)) {
				// handle 3D secure request
				require_once("lib-action/include/url/url.php");
				isset($processingParams->redirectParams) ? parse_str($processingParams->redirectParams, $redirectParams) : $redirectParams = array();
				redirect_post_raw($processingParams->redirectUrl, $redirectParams);
			}
			$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_PROCESSING");
			break;
		case PaymentTransaction::STATUS_DECLINED_CHARGE:
			$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_DECLINED");
            $this->_setInvoicesStatus($transaction);
			break;
		default:
			$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_ERRORED");
            $this->_setInvoicesStatus($transaction);
			break;
		}

		$this->smartyVars["currencyID"] = $transaction->currency_id;
		$this->smartyVars["payTotal"] = $transaction->transaction_amount;
		$this->smartyVars["invoices"] = $invoices;
		$this->smartyVars["authorizationCode"] = is_null($transaction->auth_code) ? $transaction->transaction_code : $transaction->auth_code;
        $this->smartyVars["transactionID"] = $transaction->id;
		$this->smartyVars["paymentNotes"] = $transaction->transaction_notes;

		$this->addTemplate("invoices/PayInvoicesResponse");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function paymentError($error_text)
	{
		$this->smartyVars["paymentError"] = 1;
		$this->smartyVars["paymentErrorText"] = $error_text;
		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("invoices/PayInvoicesResponse", $this->smartyVars));
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

    private function _createCreditCardBillingAccount()
    {
        CryptManager::setDefaultKey();
        $params["contactId"] = $this->_contact->id;
        $params["gatewayId"] = $this->_store->PrimaryGateway->id;
        $params["cardType"]  = bgpc("billingCardType");
        $params["cardName"] = bgpc("billingCardNameOnCard");
        $params["cardNum"] = bgpc("billingCardNum");
        $params["cardCvv2"] = bgpc("cardCVV2");
        $params["cardExpYear"] = bgpc("billingCardExpYear");
        $params["cardExpMonth"] = bgpc("billingCardExpMonth");
        $params["cardStartDate"] = bgpc("billingCardStartDate");
        $params["cardIssueNum"] = bgpc("billingCardIssueNum");
        $billingAccount = BillingAccountManager::create($params);
        BillingAccountManager::update($billingAccount);

        return $billingAccount;
    }

    private function _getBillingAccountById($id)
    {
        CryptManager::setDefaultKey();
        $params["bankAccountNumExisting"] = bgpc("billingBankAccountNumExisting-".$id);
        $params["cardNumExisting"] = bgpc("billingCardNumExisting-".$id);
        $params["cardCVV2existing"] = bgpc("cardCVV2existing-".$id);
        $billingAccount = BillingAccountManager::getBillingAccountById($id, $params);
        return $billingAccount;
    }

    private function _setInvoicesStatus(PaymentTransaction $transaction)
    {
        $status = Transaction::STATUS_DECLINED_CHARGE == $transaction->auth_return ? Invoice::STATUS_HAS_DECLINED_CHARGES : Invoice::STATUS_HAS_ERRORED_CHARGES;
        foreach($transaction->InvoiceList() as $invoice) {
            $invoice->invoice_status = $status;
            InvoiceManager::update($invoice);
        }
        return true;
    }

    private $_store = null;
    private $_contact = null;
}