<?php

require_once("lib-action/action.php");

use Billing\Manager\ContactManager;
use Billing\Object\Contact;

class AddClientContactResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$current_contact = Factory()->Contact(getCurrentSession()->user_id);

		if ( 0 == $current_contact->contact_primary ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "Only the primary contact can add contacts.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$this->setReferralActionID(getActionID("AddClientContact"));
		$this->setReferralActionFile("profile.php");
		
		if (!ContactManager::isUsernameUnique(bgpc("contactEmail"))) {
			$this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
		}
		
        $this->_validateZip();
        $this->_validateAndTransformCodeState();
        
		$this->validateMultiInputs(array(
			"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactEmail" => array(INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "contacts", "col" => "contactEmail")),
			"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactCompany" => INPUT_RESTRICTION_STRING,
			"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactAddress2" => INPUT_RESTRICTION_STRING,
			"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactState" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
			"contactPhone2" => INPUT_RESTRICTION_PHONE,
			"contactFax" => INPUT_RESTRICTION_PHONE,
			"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
			"emailActionIDs" => INPUT_RESTRICTION_ARRAY,
			"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL,
			"contactPassword" => array(INPUT_RESTRICTION_PASSWORD|INPUT_RESTRICTION_REQUIRED, array("confirm" => "contactPasswordConfirm")),
		));

		$contact = Factory()->Contact();
		$contact->client_id = $current_contact->client_id;
		$contact->contact_primary = 0;
        if (!is_null($contactUsername = bgpc("contactUsername"))) {
            $validation_fields["contactUsername"] = INPUT_RESTRICTION_USERNAME|INPUT_RESTRICTION_REQUIRED;
            if (!ContactManager::isUsernameUnique($contactUsername, $contact->id)) {
                $this->invalidParams[] = "contactUsername-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
            }
        } else if (!$contact->isPrimary() && !is_null($contactEmail = bgpc("contactEmail"))) {
            if (!ContactManager::isUsernameUnique($contactEmail, $contact->id)) {
                $this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
            }
        }
		foreach ( Contact::$attrsDescriptor as $attr_name => $attr_desc ) {
			switch ($attr_name) {
				case "client_id":
				case "contact_primary":
					continue;
				case "contact_password":
					$plain_password = bgpc("contactPassword");
					$contact->contact_password = \Billing\Manager\CryptManager::getPasswordHash($plain_password);
                    $contact->contact_password_type = \Billing\Manager\CryptManager::TYPE_CRYPT;
					$contact->plainPassword = $plain_password;
					continue;
				default:
					if ( false === ($param_name = getDBXMLCol($attr_name)) || is_null($param_value = bgpc($param_name)) ) {
						continue;
					}
					$contact->$attr_name = $param_value;
			}
		}
        if (!$contact->isPrimary() && is_null($contactUsername) && !is_null($contactEmail)) {
            $contact->contact_username = $contact->contact_email;
        }
		if ( !is_null($emailActionIDs = bgpc("emailActionIDs")) ) {
			$contact->emailActionIDs = $emailActionIDs;
		}

		try {
			ContactManager::update($contact, $contact->plainPassword);

			if ( "" != ($redirectURL = bgpc("redirectURL")) && "" != ($redirectAction = bgpc("redirectAction")) )
				redirect($redirectURL."?action=".getActionID($redirectAction)."&".bgpc("redirectArgs"));

			redirect("profile.php", getActionID("ShowClientContacts"), array("clientID" => $current_contact->client_id));
		} catch ( ProductException $e ) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
        }
		return $this->hasExecutedSuccessfully();
	}
}
