<?php

require_once("lib-action/action.php");

use Billing\Manager\ContactManager;

class EditClientContactResponse extends MB_Action
{
	public $_clientContact;

	public function __construct()
	{
		parent::__construct();

		loadConfigs("userSettings");

		// set all possible xml params here, $_REQUEST is already cleaned
		$this->params = array(
			"clientContactID",
			"contactFirstName",
			"contactLastName",
			"contactEmail",
			"contactUsername",
			"contactEmailFormat",
			"contactCompany",
			"contactAddress1",
			"contactAddress2",
			"contactCity",
			"contactState",
			"contactZip",
			"countriesID",
			"languageID",
			"contactPhone1",
			"contactPhone2",
			"contactFax",
			"contactMobilePhone",
			"emailActionIDs",
			"selectableEmailActions",
			"contactActive",
			"contactUseSpecifiedEmailRulesOnly"
		);

		/**
		 *	unset illegal key values that we don't want on the back end
		 */
		$tmp = array_flip($this->params);
		$this->unsetIllegalKeys( $tmp );
		$this->params = array_flip($tmp);
	}

	public function execute()
	{
		global $CONF;
		parent::execute();

        $contactFields = array(
            "contact_active",
            "contact_company",
            "contact_first_name",
            "contact_last_name",
            "contact_email",
            "contact_username",
            "contact_address_1",
            "contact_address_2",
            "contact_city",
            "contact_state",
            "contact_county",
            "contact_zip",
            "countries_id",
            "language_id",
            "contact_phone1",
            "contact_phone2",
            "contact_fax",
            "contact_mobile_phone",
            "contact_email_format",
            "contact_use_specified_email_rules_only",
        );

		$currentUser = oneByID("contacts", $CONF["user"]["clientContactID"]);
		$clientContactID = bgpc("clientContactID");
		$requestedClientContact = oneByID('contacts',$_REQUEST['clientContactID']);

		if (!$currentUser["contactPrimary"] && $requestedClientContact['clientContactID'] != $currentUser['clientContactID']) {
			$this->addError(TRANS_ACTIONERROR, "Only the primary contact can add/edit others' contacts.");
		} elseif (!notZero($clientContactID)) {
			$this->addError(TRANS_ACTIONERROR, "Client Contact ID is not given.");
		} elseif ($CONF['user']['id'] != $requestedClientContact['clientID']) {
			$this->addError(TRANS_ACTIONERROR, "Requested client Contact ID does not belong to logged in client.");
	    } else {

			$this->setReferralActionID(getActionID("EditClientContact"));
			$this->setReferralActionFile("profile.php");
			$postedValues = array(
				"clientContactID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactPassword" => array(INPUT_RESTRICTION_PASSWORD, array("confirm" => "contactPasswordConfirm")),
				"contactEmail" => INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED,
				"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
				"contactCompany" => INPUT_RESTRICTION_STRING,
				"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactAddress2" => INPUT_RESTRICTION_STRING,
				"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactState" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
				"contactPhone2" => INPUT_RESTRICTION_PHONE,
				"contactFax" => INPUT_RESTRICTION_PHONE,
				"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
				"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL
			);
			if ( 1 == $requestedClientContact['contactPrimary'] ) {
				$postedValues["contactUsername"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
			}

            $this->_validateZip();
            $this->_validateAndTransformCodeState();
            $this->validateMultiInputs($postedValues, array(), array(), 1);
            $contact = Factory()->Contact($clientContactID);

            if (!is_null($contactUsername = bgpc("contactUsername"))) {
                $validation_fields["contactUsername"] = INPUT_RESTRICTION_USERNAME|INPUT_RESTRICTION_REQUIRED;
                if (!ContactManager::isUsernameUnique($contactUsername, $contact->id)) {
                    $this->invalidParams[] = "contactUsername-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
                }
            } else if (!$contact->isPrimary() && !is_null($contactEmail = bgpc("contactEmail"))) {
                if (!ContactManager::isUsernameUnique($contactEmail, $contact->id)) {
                    $this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
                }
            }
            if (!is_null($contactEmail = bgpc("contactEmail")) && !ContactManager::isEmailUnique($contactEmail, $contact->id)) {
                $this->invalidParams[] = "contactEmail-" . base64_encode(INPUT_RESTRICTION_NOTINCOL);
            }

            if ("" != bgpc("contactPassword")) {
                $validation_fields["contactPassword"] = array(INPUT_RESTRICTION_PASSWORD|INPUT_RESTRICTION_REQUIRED, array("confirm" => "contactPasswordConfirm"));
                $contactFields[] = "contact_password";
            }


            try {

                foreach ( $contactFields as $attrName ) {
                    if ( false === ($paramName = getDBXMLCol($attrName)) || is_null($paramValue = bgpc($paramName)) ) {
                        continue;
                    }
                    if ("contact_password" == $attrName) {
                        $contact->$attrName = \Billing\Manager\CryptManager::getPasswordHash($paramValue);
                        $contact->contact_password_type = \Billing\Manager\CryptManager::TYPE_CRYPT;
                        $contact->plainPassword = $paramValue;
                        continue;
                    }
                    $contact->$attrName = $paramValue;
                }
                $contact->emailActionIDs = (array)bgpc("emailActionIDs");
                if (!$contact->isPrimary() && is_null($contactUsername) && !is_null($contactEmail)) {
                    $contact->contact_username = $contact->contact_email;
                }

                ContactManager::update($contact, $contact->plainPassword);
            } catch (ProductException $ex) {
                billing_put_error($ex->getMessage());
                $this->validationRedirect(array(), array(), true);
            }

            if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
                redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
            } elseif( 1 == Factory()->Contact(getCurrentSession()->user_id)->contact_primary ) {
                redirect("profile.php", getActionID("ShowClientContacts"));
            } else {
                redirect("dashboard.php", getActionID("ShowDashboard"));
            }

        }

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function unsetIllegalKeys( &$array )
	{
		global $CONF;

		if (!notNull( $this->_clientContact )) {
			$this->_clientContact = oneOf('clientContacts', array('clientContactID' => (int)$_REQUEST['clientContactID']));
			$this->_clientContact['clientContactCount'] = count(arrayFor('clientContacts', array('clientID' => (int)$_REQUEST['clientID'])));
		}

		if ($this->_clientContact['clientContactCount'] == 1) {
			unset($array['contactActive']);
		}

		foreach( (array)$CONF["userSettings"]["display"] as $k => $v ) {
			if ($v==1)
				continue;

			switch( $k ) {
			case "editUserName":
				unset($array["contactUsername"]);
				break;
			case "editFirstName":
				unset($array["contactFirstName"]);
				break;
			case "editLastName":
				unset($array["contactLastName"]);
				break;
			case "editEmail":
				unset($array["contactEmail"]);
				break;
			case "editCompanyName":
				unset($array["contactCompany"]);
				break;
			case "editCountry":
				unset($array["countriesID"]);
				break;
			case "editEmailActions":
				unset($array["emailActionIDs"]);
				unset($array["selectableEmailActions"]);
				break;
			}
		}	unset( $k, $v );

		$contact = Factory()->Contact(bgpc("clientContactID"));
		if (!ContactManager::isUsernameChangeable($contact)) {
			if ($contact->isPrimary()) {
				unset($array["contactUsername"]);
			} else {
				unset($array["contactEmail"]);
			}
		}
	}
}
