<?php

class SelectServiceAction extends CartAction
{
	public function execute($request)
	{
		sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
		$services = null;
		if ($this->hasParentPlanInCart()) {
			$cartItem = $this->_cart->getCurrentItem();
			$purchase = $cartItem->getMasterPurchase();
		}
		if ($request->isMethod('post')) {
			if ($this->hasParentPlanInCart()) {
				$planId = $purchase->getPlanId();
				$plan = ResourceLocator::getInstance()->billingPlanService->getById($planId, $this->_store->getId());
				foreach ($plan->getServices($this->_store->getId()) as $service) {
					if ($serviceId = $request->getParameter('service_' . $purchase->getId() . '_' . $service->getId())) {
						if ($purchase->hasPurchasesByPlanId($serviceId)) {
							continue;
						}
						$priceId = $request->getParameter('price_' . $service->getId());
						$cycleId = $request->getParameter('cycle_' . $service->getId());
						$purchase->addPurchase(new Purchase($serviceId, $cycleId, $priceId, $service->productName, $service->productType));
					}
				}
			} else {
				$plan = ResourceLocator::getInstance()->billingPlanService->getById($this->getUser()->getAttribute('parentPlanId'), $this->_store->getId());
				$cartItem = new CartItem($plan->id);
				foreach ($plan->getServices($this->_store->getId()) as $service) {
					if ($serviceId = $request->getParameter('service__' . $service->getId())) {
						$priceId = $request->getParameter('price_' . $service->getId());
						$cycleId = $request->getParameter('cycle_' . $service->getId());
						$cartItem->addPurchase(new Purchase($serviceId, $cycleId, $priceId, $service->productName, $service->productType, $this->getUser()->getAttribute('subscriptionId')));
					}
				}
				if(count($cartItem->getPurchases())) {
					$this->_cart->setCurrentItem($cartItem);
					$this->_cart->attachCurrentItem();
				}
			}
			$this->_router->next(array(), false);
		}
		if (!$this->hasParentPlanInCart()) {
			$plan = $this->_store->getPlan($this->getUser()->getAttribute('parentPlanId'));
			$services = $plan->getServices($this->_store->getId(), true);
			if (! $services) {
				$this->_router->next(array(), false);
			}
			$this->services = $services;
			$this->plan = $plan;
			$this->domains = array();
			$this->purchaseId = null;
			$this->selectedServices = null;
		} else {
			$selectedServices = array();
			if ($cartItem->hasMasterPurchase()) {
				$purchase = $cartItem->getMasterPurchase();
			} else {
				// No master purchase - no services
				$this->_router->next(array(), false);
			}
			$plan = $this->_store->getPlan($purchase->getPlanId(), true);
			if ($services = $plan->getServices($this->_store->getId(), true)) {
				$selectedServices[$purchase->getId()] = array();
				foreach ($services as $service) {
					if ($purchase->hasPurchasesByPlanId($service->getId())) {
						$selectedServices[$purchase->getId()][$service->getId()] = true;
					}
				}
			}
			if (! $services) {
				$this->_router->next(array(), false);
			}

			$domains = array();
			foreach (array_keys($purchase->getOwnDomains()) as $domainName) {
				$domains[$domainName] = null;
			}
			if (! ($plan instanceof BillingDomainPlan) && $purchase) {
				$plans = $this->_store->getPlans(true);
				foreach ($purchase->getPurchases() as $subPurchase) {
					$planId = $subPurchase->getPlanId();
					if ($plans[$planId]->productType == BillingPlan::TYPE_DOMAIN) {
						$data = $subPurchase->getParameter('domainData');
						$domains[$data['name']] = $subPurchase->getParameter('type');
					}
				}
			}

			$this->plan = $plan;
			$this->domains = $domains;
			$this->purchaseId = $purchase->getId();
			$this->services = $services;
			$this->selectedServices = $selectedServices;
		}
		if ($defaultTaxZoneId = isset($this->_store->defaultTaxZoneID) ? $this->_store->defaultTaxZoneID : false) {
			$this->taxZone = ResourceLocator::getInstance()->billingTaxZoneService->getById($defaultTaxZoneId);
		}
		if ($this->errorMessage = sfContext::getInstance()->getUser()->getAttribute('errorMessage')) {
			sfContext::getInstance()->getUser()->setAttribute('errorMessage', null);
		}
	}

	private function hasParentPlanInCart()
	{
		return !is_null($this->_cart->getCurrentItem());
	}
}